clc;
clear all;
close all;

%% SETUP
% simulation duration (seconds)
T = 60;

% number of equidistant spatial points to simulate at
N_points = 10;

% frequency range
omega_delta = 0.0001;
omega_max = 20;

% air density
rho = 1.25;

% model properties
V = 5;             % mean wind velocity
L = 2.64;           % model length
D = 0.065;          % depth
B = 0.376;          % width
C_D_mean = 0.749;   % drag coefficient corresponding to mean angle of incidence
C_D_slope = 0;      % drag coefficient slope evaluated at mean angle of incidence
C_L_mean = -0.254;  % drag coefficient corresponding to mean angle of incidence
C_L_slope = 2.502;  % drag coefficient slope evaluated at mean angle of incidence
C_M_mean = 0.004;   % drag coefficient corresponding to mean angle of incidence
C_M_slope = 0.859;  % drag coefficient slope evaluated at mean angle of incidence

%% INITIALISATION
points = linspace(0,1,N_points);
N_timeseries = 2*N_points; % two components at each point
omega = omega_delta:omega_delta:omega_max;
omega_resolution = length(omega);

%% FREQUENCY DOMAIN PROPERTIES OF THE WIND FIELD
G = zeros(N_timeseries,N_timeseries,omega_resolution);

% establish the cross spectral density matrix for each frequency setting
for k = 1:omega_resolution
    
    % cross spectral densities for each spatial separation
    S_dx = zeros(2,2,N_points);
    for n = 1:N_points
        dx = abs(points(1)-points(n));
        S_dx(:,:,n) = cross_spectral_densities(V,omega(k),dx);
    end
    
    % upper triangle of the (symmetric) matrix
    S_ut = zeros(N_timeseries,N_timeseries);
    for n = 1:N_points
        i = 1;
        for m = n+1:N_points
            i = i+1;
            S_ut(2*n-1:2*n,2*m-1:2*m) = S_dx(:,:,i);
        end
    end
    
    % diagonal of the matrix
    S_diag = zeros(N_timeseries,N_timeseries);
    for n = 1:N_points
        S_diag(2*n-1:2*n,2*n-1:2*n) = S_dx(:,:,1);
    end
    
    % cholesky decomposition
    S = S_ut+S_diag+S_ut';
    G(:,:,k) = chol(S,'lower');
end

%% TIME DOMAIN SIMULATION OF THE TURBULENCE COMPONENTS
N_FFT = 2^nextpow2(2*omega_resolution); % the ifft-algorithm is faster when N is a power of 2
x = zeros(N_timeseries,N_FFT);

% random phase angles
phi = 2*pi*rand(N_timeseries,omega_resolution);

% calulate each time series
for m = 1:N_timeseries
    for n = 1:m
        c = permute(G(m,n,:),[1,3,2]).*exp(1i.*phi(n,:));
        x(m,:) = x(m,:)+real(ifft(c,N_FFT))*N_FFT*sqrt(2*omega_delta);
    end
end

%% CALCULATE EQUIVALENT FORCES IN ABAQUS AND WRITE TO FILES
% assemble component matrices
N = round(N_FFT*T*omega_delta/(2*pi));                  % number of discrete points included in time series
u = x(1:2:(N_points*2-1),1:N)';
w = x(2:2:(N_points*2),1:N)';

% calculate equivalent concentrated forces and moments in Abaqus
dL = L/(N_points-1);

drag = zeros(size(u));
drag = drag+0.5*rho*V^2*D*C_D_mean;                     % static load
drag = drag+rho*V*D*C_D_mean*u;                         % direct contribution from the u-component of the turbulence
drag = drag+0.5*rho*V*B*(D/B*C_D_slope-C_L_mean)*w;     % direct contribution from the w-component of the turbulence
drag = drag*dL;                                         % transform line load into equivalent concentrated forces
drag(:,[1,N_points]) = drag(:,[1,N_points])/2;          % end forces represent half the area of what inner forces do

lift = zeros(size(w));
lift = lift+0.5*rho*V^2*B*C_L_mean;                     % static load
lift = lift+rho*V*B*C_L_mean*u;                         % direct contribution from the u-component of the turbulence
lift = lift+0.5*rho*V*B*(C_L_slope+(D/B)*C_D_mean)*w;   % direct contribution from the w-component of the turbulence
lift = lift*dL;                                         % transform pressure into equivalent concentrated forces
lift(:,[1,N_points]) = lift(:,[1,N_points])/2;          % end forces represent half the area of inner forces

moment = zeros(N,N_points);
moment = moment+0.5*rho*V^2*B^2*C_M_mean;               % static load
moment = moment+rho*V*B^2*C_M_mean*u;                   % direct contribution from the u-component of the turbulence
moment = moment+0.5*rho*V*B^2*C_M_slope*w;              % direct contribution from the w-component of the turbulence
moment = moment*dL;                                     % transform line load into equivalent concentrated forces
moment(:,[1,N_points]) = moment(:,[1,N_points])/2;      % end forces represent half the area of what inner forces do

% transform all concentrated forces into equivalent forces at the section ends
clc;
tm = [linspace(1,0,N_points)' linspace(0,1,N_points)'];
drag = drag*tm;
lift = lift*tm;

% moments are modelled as equal at section ends
tm = 0.5*ones(N_points,1);
moment = moment*tm;
%%
% write to files
data = zeros(N,1);
for i = 1:2
    data(:,1) = drag(:,i);
    save(sprintf('timeseries/simulated_wind_field_V%i_P%i_D.txt',V,i),'data','-ascii');
    data(:,1) = lift(:,i);
    save(sprintf('timeseries/simulated_wind_field_V%i_P%i_L.txt',V,i),'data','-ascii');
end
data(:,1) = moment;
save(sprintf('timeseries/simulated_wind_field_V%i_M.txt',V),'data','-ascii');

%% DISPLAY TIME SERIES PROPERTIES
fprintf('COVARIANCE MATRIX OF TURBULENCE TIME SERIES (SIMULATED FOR %i SECONDS):\n',T);
fprintf('u-components (mean: %.1f):\n',sum(mean(u)));
disp(cov(u));
fprintf('w-components (mean: %.1f):\n',sum(mean(w)));
disp(cov(w));
fprintf('COVARIANCE MATRIX OF FORCE TIME SERIES (SIMULATED FOR %i SECONDS):\n',T);
fprintf('Drag (mean: %.1f):\n',sum(mean(drag)));
disp(cov(drag));
fprintf('Lift (mean: %.1f):\n',sum(mean(lift)));
disp(cov(lift));
fprintf('Moment (mean: %.1f):\n',2*mean(moment));
disp(cov(moment));