%   IDENTIFICATION_EXAMPLE Shows how implemented methods can be used in
%   practice. This example do not involve static displacement correction.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc;
clear all;
close all hidden;

%% INPUT
% data source
MEASUREMENTS = 1;                                                   % source 1: measured in experiments
ABAQUS = 2;                                                         % source 2: abaqus data file (U.xls)
SIMULATION = 3;                                                     % source 3: simulated by finite differences
source = MEASUREMENTS;

% analysis settings
ssOrder = 10;
fsAnalysis = 10;                                                    % sampling frequency used in analysis (measurements will be resampled to this sampling frequency)
cbhmMaxLag = fsAnalysis/0.68;                                       %
activeModes = [1,2,3,4,5];                                          % active (excited) modes
range = [0,60];                                                     %

% measurement data
measurementFileName = 'AmbientFreeVibration1';                      %

% abaqus data
if source == ABAQUS
    abaqusTestName = 'BigHorizontalAmplitude';                      %
    abaqusFs = 100;                                                 %
    abaqusRNS = 0;                                                  % noise-to-signal ratio applied when contaminating signal
end

% simulated data
if source == SIMULATION
    simulationInitialDisplacements = [0.10 0.05 0.06 0.05 0.20]';   % excitation in meters (translational dofs) or radians (rotational dofs)
    simulationFs = 200;                                             %
    simulationRNS = 0.01;                                           % noise-to-signal ratio applied when contaminating signal
end

%% SETTINGS
printSummary = true;
showPlots = true;
showComparisons = false;

%% SOFTWARE CHECK
warning('on','IDENTIFICATION:softwareVersion');
software = {                                                        % required toolboxes and recommended versions
    'MATLAB',7.14
    'Signal Processing Toolbox',6.17
    'System Identification Toolbox',8.0
    };
v = ver;
for i = 1:size(software,1)
    exists = false;
    for j = 1:numel(v)
        if strcmpi(v(j).Name,software{i,1})
            if str2double(v(j).Version) < software{i,2}
                warning('IDENTIFICATION:softwareVersion','The installed version of %s (%s) is older than the version used during developement of the function (%g), and hence it may not work properly.',software{i,1},v(j).Version,software{i,2});
            end
            exists = true;
            break;
        end
    end
    if ~exists
        error('IDENTIFICATION:missingSoftware','Missing software: %s',software{i,1});
    end
end
warning('off','IDENTIFICATION:softwareVersion');

%% ADD DATA FOLDERS TO PATH
addpath('abaqus','measurements');

%% LOAD DATA
% rig properties
Props = getrigproperties();

if source == MEASUREMENTS
    % load measured data
    [displacements,fs] = loaddata(['measurements\' measurementFileName],range(1),range(2),Props);
    
elseif source == ABAQUS
    % get abaqus model properties
    Props = getabaqusproperties();
    
    % load abaqus displacements
    displacements = loadabaqusdata(['abaqus\' abaqusTestName],abaqusFs,range(1),range(2));
    
    % contaminate signals
    if abaqusRNS > 0
        [displacements,appliedRNS] = contaminatesignal(displacements,abaqusRNS);
        fprintf('Applied RNS: %.1f %%\n',appliedRNS*100+1e-99);     % add an extremely small value to avoid trouble printing 0
    end
    
    % sampling frequency
    fs = abaqusFs;
    
elseif source == SIMULATION
    % simulate displacements numerically
    displacements = simulatedisplacements(range,simulationFs,simulationInitialDisplacements,Props.M,Props.C,Props.K);
    
    % contaminate signals
    if simulationRNS > 0
        [displacements,appliedRNS] = contaminatesignal(displacements,simulationRNS);
        fprintf('Applied RNS: %.1f %%\n',appliedRNS*100+1e-99);     % add an extremely small value to avoid trouble printing 0
    end
    displacements = displacements(1:range(2)*simulationFs+1,:);
    
    % sampling frequency
    fs = simulationFs;
end

%% SYSTEM IDENTIFICATION
% identify state-space system matrices
[exactA,cbhmA,n4sidA,r] = identification(1/fs,displacements,activeModes,Props.M,Props.C,Props.K,...
    'fsAnalysis',fsAnalysis,'ssOrder',ssOrder,'cbhmMaxLag',cbhmMaxLag,'showPlots',showPlots,'showComparisons',showComparisons);

% extract structural properties
exactM = Props.M(activeModes,activeModes);
exactC = Props.C(activeModes,activeModes);
exactK = Props.K(activeModes,activeModes);
[~,~,exactFrequency,~,exactZeta] = extractstructuralproperties(exactA,exactM);
[cbhmK,cbhmC,cbhmFrequencyA,cbhmFrequencyK,cbhmZeta] = extractstructuralproperties(cbhmA,exactM);
[n4sidK,n4sidC,n4sidFrequencyA,n4sidFrequencyK,n4sidZeta,modeShapes] = extractstructuralproperties(n4sidA,exactM);

%% COMPARISON PRINT
if printSummary
    fprintf('***************************************************************************\n');
    fprintf('COMPARISON OF SYSTEM IDENTIFICATION METHODS\n');
    fprintf('***************************************************************************\n\n');
    
    % stiffness matrix
    fprintf('****** STIFFNESS MATRIX ***************************************************\n\n');
    fprintf('Estimated stiffness matrix (CBHM):\n');
    disp(cbhmK);
    fprintf('Estimated stiffness matrix (N4SID):\n');
    disp(n4sidK);
    fprintf('Exact stiffness matrix:\n');
    disp(exactK);
    
    % damping matrix
    fprintf('****** DAMPING MATRIX *****************************************************\n\n');
    fprintf('Estimated damping matrix (CBHM):\n');
    disp(cbhmC);
    fprintf('Estimated damping matrix (N4SID):\n');
    disp(n4sidC);
    fprintf('Exact damping matrix:\n');
    disp(exactC);
    
    % natural frequencies
    fprintf('****** UNDAMPED NATURAL FREQUENCIES ***************************************\n\n');
    printvectors(...
        'header','Estimated natural frequencies [Hz] (origin matrix given in paranthesis):',...
        'CBHM(A)',cbhmFrequencyA,...
        'CBHM(K)',cbhmFrequencyK,...
        'N4SID(A)',n4sidFrequencyA,...
        'N4SID(K)',n4sidFrequencyK,...
        'EXACT',exactFrequency);
    printvectors(...
        'header','Error in estimated natural frequencies [%%] (origin matrix given in paranthesis):',...
        'includeAbsMean',true,...
        'CBHM(A)',(cbhmFrequencyA./exactFrequency-1)*100,...
        'CBHM(K)',(cbhmFrequencyK./exactFrequency-1)*100,...
        'N4SID(A)',(n4sidFrequencyA./exactFrequency-1)*100,...
        'N4SID(K)',(n4sidFrequencyK./exactFrequency-1)*100);
    
    % modal damping ratios
    fprintf('****** MODAL DAMPING RATIOS ***********************************************\n\n');
    printvectors(...
        'header','Estimated modal damping ratios:',...
        'CBHM',cbhmZeta*100,...
        'N4SID',n4sidZeta*100,...
        'EXACT',exactZeta*100);
    printvectors(...
        'header','Error in estimated modal damping ratios [%%]:',...
        'includeAbsMean',true,...
        'CBHM',(cbhmZeta./exactZeta-1)*100,...
        'N4SID',(n4sidZeta./exactZeta-1)*100);
end

%% COMPARISON PLOTS
if showPlots
    plotmatrixcomparison(exactK,...
        'figName','Stiffness matrix',...
        'legendPositiveReference','Exact values',...
        'legendPositiveValue','Estimated values (positive)',...
        'legendNegativeValue','Estimated values (negative)',...
        'Stiffness matrix (CBHM)',cbhmK,...
        'Stiffness matrix (N4SID)',n4sidK);
    plotmatrixcomparison(exactC,...
        'figName','Damping matrix',...
        'legendPositiveReference','Exact values',...
        'legendPositiveValue','Estimated values (positive)',...
        'legendNegativeValue','Estimated values (negative)',...
        'Damping matrix (CBHM)',cbhmC,...
        'Damping matrix (N4SID)',n4sidC);
    plotvectorcomparison(exactFrequency,...
        'figName','Undamped natural frequencies',...
        'valueLabel','Frequency (Hz)',...
        'maxMinValue',0,...
        'legendReference','Exact values',...
        'legendValue','Estimated values',...
        'CBHM (A)',cbhmFrequencyA,...
        'CBHM (K)',cbhmFrequencyK,...
        'N4SID (A)',n4sidFrequencyA,...
        'N4SID (K)',n4sidFrequencyK);
    plotvectorcomparison(exactZeta*100,...
        'figName','Modal damping ratios',...
        'valueLabel','Damping ratio (%)',...
        'legendReference','Exact values',...
        'legendValue','Estimated values',...
        'CBHM',cbhmZeta*100,...
        'N4SID',n4sidZeta*100);
end