function displacements = loadabaqusdata(filename,fs,startTime,endTime)
%   LOADABAQUSUDATA Loads U-data from Abaqus and transform it into displacements of system DOFs.
%
%   INPUT
%       filename
%           Input xls file containing displacements from an abaqus
%           simulation. Columns:
%               1. Time
%               2. U:U1 PI: SUSPENSION-1 N:2
%               3. Time
%               4. U:U2 PI: SUSPENSION-2 N:2
%               5. Time
%               6. UR:UR1 PI: SUSPENSION-1 N:2
%               7. Time
%               8. UR:UR2 PI: SUSPENSION-1 N:2
%               9. Time
%              10. UR:UR3 PI: SUSPENSION-1 N:2
%       fs
%           Sampling frequency. The (unequal spaced) samples from Abaqus are
%           interpolated and resampled at this frequency.
%       startTime
%           Start position of the time window to be loaded, given in
%           seconds.
%       endTime
%           End position of the time window to be loaded, given in
%           seconds. If the position is beyond the end of the time series,
%           the window will end there too.
%
%   OUPUT
%       Displacement of system DOFs. Columns:
%           1. Horizontal translation
%           1. Horizontal rotation
%           1. Vertical translation
%           1. Vertical rotation
%           1. Longitudinal (torsional) rotation

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% LOAD AND PROCESS U-DATA
data = xlsread(filename);
t = data(:,1)-data(1,1);                                    % time shift (to start at t = 0 seconds)
data = data(:,2:2:size(data,2));                            % remove time columns
data = interp1(t,data,t(1):1/fs:t(end),'spline');           % interpolate to achieve evenly spaced samples
size(data)

% extract data
startPos = floor(startTime*fs+1);
endPos = ceil(endTime*fs+1);
if endPos > size(data,1)
    endPos = size(data,1);
end
displacements = data(startPos:endPos,[1,4,2,3,5]);

end
