function [A] = cbhm(d,dt,order,maxLag,N,varargin)
%   CBHM Estimates the system matrix A by the Covariance Block Hankel Matrix
%   method.
%
%   REQUIRED INPUT
%       d
%           Displacement time series, where each column constitute the
%           displacements of a modal degree of freedom.
%       dt
%           Sampling interval.
%       order
%           State space model order.
%       maxLag
%           Maximum lag for covariance estimates.
%       N
%           Normalization matrix that has been used to normalize time
%           series BEFORE this method is invoked. If they have not been
%           normalized, N should be the identity matrix.
%
%   OPTIONAL PARAMETERS
%       Key                   | Value
%       -------------------------------------------------------------------
%       'plotSingularValues'  | Whether or not to plot singular values
%                             |     of the covariance block Hankel matrix.
%
%   OUPUT
%       A
%           Dynamic system matrix of the state space model, transformed to
%           obtain the special structure required for extraction of
%           structural matrices.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% DEFAULT SETTINGS
plotSingularValues = false;

%% PARSE AND VALIDATE INPUT
% validation functions
isbool = @(x) islogical(x) && isscalar(x);
isrealscalar = @(x) ~ischar(x) && isreal(x) && isscalar(x);
isnonnegativescalar = @(x) isrealscalar(x) && x > 0;
isrealmatrix = @(x) ismatrix(x) && isnumeric(x) && isreal(x);

% build input scheme
p = inputParser;
p.FunctionName = 'cbhm';
p.CaseSensitive = false;
p.addRequired('d',isrealmatrix);
p.addRequired('dt',isnonnegativescalar);
p.addRequired('order',isnonnegativescalar);
p.addRequired('maxLag',isnonnegativescalar);
p.addRequired('N',isrealmatrix);
p.addParamValue('plotSingularValues',plotSingularValues,isbool);
p.parse(d,dt,order,maxLag,N,varargin{:});

% get parameters
plotSingularValues = p.Results.plotSingularValues;
[nSamples,nDofs] = size(d);

%% COVARIANCE BLOCK HANKEL MATRIX
lastwarn('');
cbhmTimer = tic;
maxLag = min(maxLag,nSamples-1);                            % maximum time lag is restricted by number of samples in time series
l = ceil(maxLag/2);                                         % size of block hankel matrix

% covariance blocks
Cyy = cell(2*l,1);
progressStr = 'CBHM progress: ';
backspaces = [];
for k = 1:2*l
    Cyy{k} = zeros(nDofs);
    for i = 1:(nSamples-k)
        Cyy{k} = Cyy{k}+d(i+k,:)'*d(i,:);
    end
    Cyy{k} = Cyy{k}/nSamples;
    
    % update progress
    str = [progressStr sprintf('%i %%%%',round(k/(2*l)*99))];
    fprintf([backspaces str]);
    backspaces = repmat(sprintf('\b'),1,length(str)-1);
end

% block hankel matrix combining two shifts (memory efficient)
H = zeros(nDofs*(l+1),nDofs*l);
for k = 1:l
    for i = 1:k
        row = i;
        col = k-i+1;
        H((row-1)*nDofs+1:row*nDofs,(col-1)*nDofs+1:col*nDofs) = Cyy{k};
    end
end
for k = l+1:2*l
    for i = 1:2*l-k+1
        row = k-l+i;
        col = l-i+1;
        H((row-1)*nDofs+1:row*nDofs,(col-1)*nDofs+1:col*nDofs) = Cyy{k};
    end
end
clear d dCov;                                               % free redundant memory

%% SYSTEM MATRIX
% singular value decomposition of CBHM shifted one step.
% only the first (2*n_dofs) values are required to form the system matrix.
[U,S,V] = svd(H(1:end-nDofs,:));
U2 = U;%%%
S2 = S;%%%
V2 = V;%%%
H2 = H(nDofs+1:end,:);%%%
sv = diag(S);
U = U(:,1:order);
S = sqrt(S(1:order,1:order));
V = V(:,1:order);

% discrete matrices F and C
F = S\U'*H(nDofs+1:end,:)*V/S;
C = [eye(nDofs) zeros(nDofs,(l-1)*nDofs)]*U*S;
C = N\C;                                                    % signals were normalized to unit variances by the normalization matrix N before identification
clear H;                                                    % free redundant memory

% continuous matrix A
[eigenVectorsF,eigenValuesF] = eig(F);
eigenValuesA = log(diag(eigenValuesF))/dt;
A = real(eigenVectorsF*diag(eigenValuesA)/eigenVectorsF);
invT = [C; C*A];
A = invT*A/invT;                                            % transform to required structure

%% PLOT SINGULAR VALUES
if plotSingularValues
    maxOrder = min(order*4,numel(sv)-1);
    rsv = sv(1:maxOrder)./sv(2:maxOrder+1);
    plotsignal(1,...
        'figName','Singular values obtained by the CBHM method',...
        'channelTitle',{'Singular values','Relative singular values'},...
        'channelYLabel',{'SV, s_i','Rel. SV, s_i/(s_i+1)'},...
        'tLabel','Model order',...
        't0',1,...
        'includePsd',false,...
        'includeLegend',false,...
        'precision',[0 0],...
        'optimizeYAxes',true,...
        'equalMaxMin',false,...
        'Singular values',[sv(1:maxOrder) rsv],'color','r');
end

%% PRINT SUMMARY
if isempty(lastwarn())
    fprintf(backspaces);
end
fprintf('CBHM identification performed in %i ms\n\n',round(toc(cbhmTimer)*1000));