function [contaminatedSignal,appliedRns] = contaminatesignal(signal,targetRns)
%   CONTAMINATESIGNAL Contaminates a signal with white noise.
%
%   INPUT
%       signal
%           Matrix containing discrete time series. Each column corresponds
%           to a signal channel.
%       targetRns
%           Target noise-to-signal ratio.
%
%   OUPUT
%       contaminatedSignal
%           Matrix containing the sum of the input data and the generated
%           noise.
%       appliedRns
%           Measurement of the noise-to-signal ratio AFTER the noise has
%           been applied. Due to the stochastic nature of the noise, this
%           value is not necessarily equal to the target value.
%
%   EXAMPLE
%       rns = 0.1;
%       t = 0:0.001:1;
%       y = sin(2*pi*1.5*t)';
%       contaminatedY = contaminatesignal(y,rns);
%       plot(t,contaminatedY,t,y);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% CONTAMINATE
% signal properties
nSamples = size(signal,1);
nChannels = size(signal,2);
signalPower = sum(detrend(signal).^2)/nSamples;             % non-zero mean may give rise to massive noise

% generate noise with required average RNS
noisePower = signalPower*targetRns;
noise = randn(nSamples,nChannels)*diag(sqrt(noisePower));
appliedRns = sum(noise.^2)/nSamples/signalPower;

% apply noise
contaminatedSignal = signal+noise;