function [K,C,fA,fK,zeta,modeShapes] = extractstructuralproperties(A,M)
%   EXTRACTSTRUCTURALPROPERTIES Extracts stiffness and damping matrices
%   from the state-space system matrix A, given the mass matrix M.
%
%   INPUT
%       A
%           Dynamic system matrix of the state space model, transformed to
%           the special structure required for extraction of structural
%           matrices.
%       M
%           Modal mass matrix of the system.
%
%   OUPUT
%       K
%           Modal stiffness matrix.
%       C
%           Modal damping matrix.
%       fA
%           Natural frequencies extracted from the system matrix A.
%       fK
%           Natural frequencies extracted from the modal stiffness matrix K.
%       zeta
%           Modal damping ratios extracted from the system matrix A.
%       modeShapes
%           Complex mode shapes extracted from the system matrix A.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% EXTRACT STIFFNESS AND DAMPING MATRIX (ASSUMING CORRECT MASS MATRIX)
nDofs = size(A,1)/2;
K = -M*A(nDofs+1:end,1:nDofs);
C = -M*A(nDofs+1:end,nDofs+1:end);

%% ESTIMATE FREQUENCIES AND DAMPING RATIOS
[vectors,values] = eig(A);
values = diag(values);
[~,sortedPos] = sort(imag(values),'ascend');
values = values(sortedPos(end/2+1:end));

zeta = -real(values)./abs(values);
fA = abs(values)/(2*pi);

% estimate frequencies from stiffness matrix as well 
[~,values] = eig(K,M);
fK = sqrt(sort(diag(values),'ascend'))/(2*pi);

%% ESTIMATE DAMPED MODES
modeShapes = vectors(end/2+1:end,sortedPos(end/2+1:end));
for i = 1:nDofs
    [m,pos] = max(abs(modeShapes(:,i)));
    modeShapes(:,i) = modeShapes(:,i)/m*sign(real(modeShapes(pos,i)));
end