function [exactA,cbhmA,n4sidA,meanDisplacements] = identification(dt,displacements,activeModes,exactM,exactC,exactK,varargin)
%   IDENTIFICATION Estimates structural matrices with various system
%   identification methods.
%
%   REQUIRED INPUT
%       dt
%           Sampling interval.
%       displacements
%           Displacement time series, where each column constitute the
%           displacements of a modal degree of freedom.
%       activeModes
%           Modes included in the identificaton, specified by a vector
%           included their respective numbers. All active modes must be
%           excited.
%       exactM
%           Exact (reference) modal mass matrix. Used as reference and to
%           determine cutoff frequency.
%       exactC
%           Exact (reference) modal damping matrix. Used as reference.
%       exactK
%           Exact (reference) modal stiffness matrix. Used as reference to
%           determine cutoff frequency.
%
%   OPTIONAL PARAMETERS
%       Key                   | Value
%       -------------------------------------------------------------------
%       'fsAnalysis'          | Sampling frequency used in analysis -
%                             |     measurements will be resampled to
%                             |     this sampling frequency.
%       'filterOrder'         | Effective order (multiplum of 2) of low-pass
%                             |     butterworth filter.
%       'filterCutoffFactor'  | Butterworth filter cut-off frequency as a
%                             |     ratio of the estimated maximum natural
%                             |     frequency.
%       'windowChopFactor'    | Chop off this part of the time series at
%                             |     both ends (inaccurate data due to
%                             |     filtering/resampling).
%       'cbhmMaxLag'          | Maximum time lag (steps) included in
%                             |     covariance estimation in the CBHM method.
%       'ssOrder'             | Order of state space system matrix.
%       'showPlots'           | Wheter or not to show relevant plots.
%       'showComparisons'     | Wheter or not to show comparison of signal
%                             |     before and after filtering.
%
%   OUPUT
%       exactA
%           Exact (reference) state space system matrix.
%       cbhmA
%           State space system matrix identified by the CBHM method.
%       n4sidA
%           State space system matrix identified by the N4SID method.
%       meanDisplacements
%           Mean (static) displacement of each modal degree of freedom.
%           Can be used for correction of static displacement effects.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% SOFTWARE CHECK
software = {                                                % required toolboxes and recommended versions
    'MATLAB',7.14
    'Signal Processing Toolbox',6.17
    'System Identification Toolbox',8.0
    };
v = ver;
for i = 1:size(software,1)
    exists = false;
    for j = 1:numel(v)
        if strcmpi(v(j).Name,software{i,1})
            if str2double(v(j).Version) < software{i,2}
                warning('IDENTIFICATION:softwareVersion','The installed version of %s (%s) is older than the version used during developement of the function (%g), and hence it may not work properly.',software{i,1},v(j).Version,software{i,2});
            end
            exists = true;
            break;
        end
    end
    if ~exists
        error('IDENTIFICATION:missingSoftware','Missing software: %s',software{i,1});
    end
end

%% DEFAULT SETTINGS
fsAnalysis = 10;
filterOrder = 16;
filterCutoffFactor = 1.5;
windowChopFactor = 0.1;
cbhmMaxLag = 2*fsAnalysis;
ssOrder = numel(activeModes)*2;
showPlots = true;
showComparisons = false;

%% PARSE AND VALIDATE INPUT
% validation functions
isbool = @(x) islogical(x) && isscalar(x);
isrealscalar = @(x) ~ischar(x) && isreal(x) && isscalar(x);
isnonnegativescalar0 = @(x) isrealscalar(x) && x >= 0;
isnonnegativescalar = @(x) isrealscalar(x) && x > 0;
isnonnegativevector5 = @(x) isvector(x) && all(arrayfun(isnonnegativescalar,x)) && numel(x) <= 5;
isrealmatrix = @(x) ismatrix(x) && isnumeric(x) && isreal(x);
isrealmatrix5 = @(x) isrealmatrix(x) && size(x,1) == 5 && size(x,2) == 5;

% build input scheme
p = inputParser;
p.FunctionName = 'identification';
p.CaseSensitive = false;
p.addRequired('dt',isnonnegativescalar);
p.addRequired('displacements',isrealmatrix);
p.addRequired('activeModes',isnonnegativevector5);
p.addRequired('M_exact',isrealmatrix5);
p.addRequired('C_exact',isrealmatrix5);
p.addRequired('K_exact',isrealmatrix5);
p.addParamValue('fsAnalysis',fsAnalysis,isnonnegativescalar);
p.addParamValue('filterOrder',filterOrder,isnonnegativescalar);
p.addParamValue('filterCutoffFactor',filterCutoffFactor,isnonnegativescalar);
p.addParamValue('windowChopFactor',windowChopFactor,isnonnegativescalar0);
p.addParamValue('cbhmMaxLag',cbhmMaxLag,isnonnegativescalar);
p.addParamValue('ssOrder',ssOrder,isnonnegativescalar);
p.addParamValue('showPlots',showPlots,isbool);
p.addParamValue('showComparisons',showComparisons,isbool);
p.parse(dt,displacements,activeModes,exactM,exactC,exactK,varargin{:});

% get parameters
fsAnalysis = p.Results.fsAnalysis;
filterOrder = p.Results.filterOrder;
filterCutoffFactor = p.Results.filterCutoffFactor;
windowChopFactor = p.Results.windowChopFactor;
cbhmMaxLag = p.Results.cbhmMaxLag;
ssOrder = p.Results.ssOrder;
showPlots = p.Results.showPlots;
showComparisons = p.Results.showComparisons;

% validate parameters
if filterCutoffFactor < 1
    warning('IDENTIFICATION:filterCutoffFactor','The Butterworth cut-off frequency will be lower than the highest estimated frequency (filterCutoffFactor=%g)',filterCutoffFactor);
end
if windowChopFactor < 0
    warning('IDENTIFICATION:windowChopFactor','The time series chop factor (%g) is negative and will be set to 0.',windowChopFactor);
elseif windowChopFactor >= 0.5
    error('IDENTIFICATION:windowChopFactor','The time series chop factor (%g) has to be within the interval [0,0.5).',windowChopFactor);
end
if ssOrder < numel(activeModes)*2
    ssOrder = numel(activeModes)*2;
    warning('IDENTIFICATION:ssOrder','The state space order has been changed to two times the number of active modes (%g). ',ssOrder);
end

%% CHANNEL INFORMATION
channelTitle = {
    'Horizontal translation'
    'Horizontal rotation'
    'Vertical translation'
    'Vertical rotation'
    'Longitudinal (torsional) rotation'
    };
channelYLabel = {
    'Displ. (mm)'
    'Rot. (rad)'
    'Displ. (mm)'
    'Rot. (rad)'
    'Rot. (rad)'
    };
channelMultiplier = diag([1000 1 1000 1 1]);
channelPrecision = [0 3 0 3 3];

%% DISCARD INACTIVE MODES
nDofs = numel(activeModes);
displacements = displacements(:,activeModes);
exactM = exactM(activeModes,activeModes);
exactC = exactC(activeModes,activeModes);
exactK = exactK(activeModes,activeModes);
channelTitle = channelTitle(activeModes);
channelYLabel = channelYLabel(activeModes);
channelMultiplier = channelMultiplier(activeModes,activeModes);
channelPrecision = channelPrecision(activeModes);

%% PREPROCESS MEASURED DISPLACEMENT TIME SERIES
% detrend signals
meanDisplacements = mean(displacements);
originalDisplacements = detrend(displacements);             % should be performed before resampling to get zero mean signals
nSamples = size(displacements,1);                           % total number of samples in each signal channel
nyq = 1/dt/2;                                               % nyquist frequency

% estimate maximum natural frequency
[~,eigenValues] = eig(exactK,exactM);
maxFrequencyEstimate = sqrt(max(diag(eigenValues)))/(2*pi);
if nyq < maxFrequencyEstimate
    warning('SYSID:nyq','Estimated maximum frequency is higher than the Nyquist frequency');
end

% low-pass chebyshev type I filter
[B,A] = cheby1(filterOrder/2,0.05,maxFrequencyEstimate/nyq*filterCutoffFactor);
filteredDisplacements = filtfilt(B,A,originalDisplacements);    % filter forward and backward to avoid phase shift

% compare original and filtered displacements
if showComparisons
    original = iddata(originalDisplacements,[],dt);
    filtered_vs_original = iddata(filteredDisplacements,[],dt);
    figure();
    compare(original,filtered_vs_original);
end

% estimate average noise-to-signal ratio
filteredPower = sum(filteredDisplacements.^2)/nSamples;
noisePower = sum((originalDisplacements-filteredDisplacements).^2)/nSamples;
estimatedRNS = noisePower./filteredPower;
disp('Estimated RNS:');
for i = 1:nDofs
    str = blanks(max(cellfun(@length,channelTitle))+1);
    str(1:length(channelTitle{i})+1) = [channelTitle{i} ':'];
    fprintf('\t%s\t%.1f %%\n',str,estimatedRNS(i)*100+1e-99);       % add an extremely small value to avoid trouble printing 0
end
fprintf('\n');

% resample (decimate) signal
fsMeasurements = 1/dt;
nSamples = ceil(nSamples*fsAnalysis/fsMeasurements);
if fsAnalysis ~= fsMeasurements
    resampledDisplacements = resample(filteredDisplacements,fsAnalysis,fsMeasurements);
else
    resampledDisplacements = filteredDisplacements;
end
dt = 1/fsAnalysis;

% to avoid inaccuracies at extreme ends of series, samples at both ends will
% be discarded - the number of which is determined by "windowChopFactor"
window = round(1+windowChopFactor*nSamples):round((1-windowChopFactor)*nSamples);
resampledDisplacements = resampledDisplacements(window,:);
nSamples = numel(window);

% plot displacement that will be used for identification
if showPlots
    plotsignal(dt,...
        'figName','Preprocessed displacements and rotations of system DOFs',...
        'channelTitle',channelTitle,...
        'channelYLabel',channelYLabel,...
        'cutoffFrequency',maxFrequencyEstimate*filterCutoffFactor,...
        'precision',channelPrecision,...
        'optimizeYAxes',true,...
        'equalMaxMin',true,...
        'Preprocessed displacements',resampledDisplacements*channelMultiplier,'color','r');
end

% normalize signals to unit variance
N = diag(1./std(resampledDisplacements));
resampledDisplacements = resampledDisplacements*N;

% free memory that are no longer needed
clear dOriginal window;

%% SYSTEM IDENTIFICATION
% exact system matrix (for comparison purposes only)
exactA = [zeros(nDofs),eye(nDofs);-exactM\exactK,-exactM\exactC];

% identification by CBHM method
cbhmA = cbhm(resampledDisplacements,dt,ssOrder,cbhmMaxLag,N,'plotSingularValues',showPlots);

% identification by a subspace method
tic;
str = 'N4SID is running...';
fprintf(str);
w = 'Ident:estimation:n4sidCheck10';
warning('off',w);                                           % turn off warning regarding n4sid "K-matrix" during identification
data = iddata(resampledDisplacements,[],dt);
ssModel = d2c(n4sid(data,ssOrder));
warning('on',w);                                            % turn warning back on
clear data;
ssModel.C = N\ssModel.C;                                    % signals were normalized to unit variances by the normalization matrix N before identification
invT = [ssModel.C; ssModel.C*ssModel.A];                    % transform to required matrix structure
n4sidA = invT*ssModel.A/invT;
fprintf([repmat(sprintf('\b'),1,length(str)) 'N4SID identification performed in %i ms\n\n'],round(toc*1000));
%#ok<*NASGU>