function Props = getrigproperties()
%   GETRIGPROPERTIES All information about the suspension rig should be
%   provided by the user in this function.
%
%   INPUT
%       None
%
%   OUTPUT
%       Structure array containing all relevant information about the
%       suspension rig.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% SETTINGS
print = false;

%% DIMENSIONS
Props = struct();

% inner height of rig, i.e. length between vertical wire mounts
Props.rigHeight = 3.050;

% inner width of rig, i.e. length between horizontal wire mounts
Props.rigWidth = 2.050;

% length of section model (excluding extension arms)
Props.sectionLength = 2.640;

% distance from section centre of mass to vertical spring mounts
Props.distanceCentreOfMassToVSprings = 1.440;

% distance from section centre of mass to horizontal spring mounts
Props.distanceCentreOfMassToHSprings = 1.490;

% distance from section longitudinal axis to vertical spring mounts
Props.distanceLongitudinalAxisToVSprings = 0.288;

% diameter of wire brackets on rotation arm
Props.wireBracketDiameter = 0.040;

%% WIRE
% diameter
Props.wireDiameter = 0.0004;

% young's modulus
Props.wireE = 208.5e9;

%% SPRINGS
% lengths
Props.hSpringLength = 0.320;
Props.uvSpringLength = 0.200;
Props.lvSpringLength = 0.090;

% stiffness
Props.hSpringStiffness = [
    10.1    % H1
    10.0    % H2
    10.0    % H3
    10.1    % H4
    ];
Props.uvSpringStiffness = [
    204.5   % UV1
    203.3   % UV2
    202.2   % UV3
    203.3   % UV4
    ];
Props.lvSpringStiffness = [
    204.4   % LV1
    211.8   % LV2
    204.0   % LV3
    205.0   % LV4
    ];

% constant forces
Props.hConstantSpringForce = [
    1.8     % H1
    1.8     % H2
    1.8     % H3
    1.7     % H4
    ];
Props.uvConstantSpringForce = [
    10.9    % UV1
    10.7    % UV2
    10.5    % UV3
    10.3    % UV4
    ];
Props.lvConstantSpringForce = [
    6.7     % LV1
    4.4     % LV2
    4.5     % LV3
    4.5     % LV4
    ];

%% INITIAL FORCES
Props.initialForce = [
    7.17    % H1
    7.10    % H4
    24.1    % LV1
    23.6    % LV2
    24.0    % LV3
    23.5    % LV4
    ];

%% DAMPING
modalDampingRatios = [
    0.06
    0.09
    0.19
    0.12
    0.25
    ]/100;

%% MASS
% suspended mass (total mass minus full spring masses, but including effective vertical spring masses)
Props.suspendedMass = 14.460;

% deviation of centre of gravity from shear centre
Props.cgDeviationX = 0.000;
Props.cgDeviationZ = 0.000;

% mass matrix (SHOULD BE UPDATED IF DISTANCE BETWEEN VERTICAL WIRE CHANGES)
M = [
    14.337  0       0       0       0.0287
    0       12.534  0       0       0
    0       0       14.460  0       0
    0       0       0       12.615  0
    0.0287  0       0       0       0.2012
    ];

%% MODAL MATRICES
% wire length estimates
hPretensionLengthEstimate = (mean(Props.initialForce(1:2))-mean(Props.hConstantSpringForce))/mean(Props.hSpringStiffness);
uvPretensionLengthEstimate = (mean(Props.initialForce(3:6))+Props.suspendedMass*9.81/4-mean(Props.uvConstantSpringForce))/mean(Props.uvSpringStiffness);
lvPretensionLengthEstimate = (mean(Props.initialForce(3:6))-mean(Props.lvConstantSpringForce))/mean(Props.lvSpringStiffness);
Props.hWireLength = (Props.rigWidth-Props.wireBracketDiameter)/2-hPretensionLengthEstimate-Props.hSpringLength;
Props.uvWireLength = (Props.rigHeight-Props.wireBracketDiameter)/2-uvPretensionLengthEstimate-Props.uvSpringLength;
Props.lvWireLength = (Props.rigHeight-Props.wireBracketDiameter)/2-lvPretensionLengthEstimate-Props.lvSpringLength;

% effective stiffness
EA = Props.wireE*pi*Props.wireDiameter^2/4;
Props.hEffectiveStiffness = 1./(Props.hWireLength/EA+1./Props.hSpringStiffness);
Props.uvEffectiveStiffness = 1./(Props.uvWireLength/EA+1./Props.uvSpringStiffness);
Props.lvEffectiveStiffness = 1./(Props.lvWireLength/EA+1./Props.lvSpringStiffness);

% stiffness matrix
K = stiffnessmatrix([0,0,0,0,0],Props,false);

% modal matrices
[vectors,values] = eig(K,M);
values = diag(values);
[values,sortedPos] = sort(values,'ascend');
vectors = vectors(:,sortedPos);
Props.M = vectors'*M*vectors;
Props.K = vectors'*K*vectors;
for i = 1:5
    [m,pos] = max(abs(vectors(:,i)));
    vectors(:,i) = vectors(:,i)/m*sign(real(vectors(pos,i)));
end
Props.C = diag(2*modalDampingRatios.*ones(5,1)).*sqrt(Props.M.*Props.K);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% PRINT
if print
    % modal structural matrices
    disp('Modal mass matrix:');
    disp(Props.M);
    disp('Modal stiffness matrix (no static displacements):');
    disp(Props.K);
    disp('Modal damping matrix:');
    disp(Props.C);
    
    % mode shapes and frequencies
    disp('Undamped mode shapes (no static displacements):');
    disp(vectors);
    disp('Natural frequencies (no static displacements):');
    for i = 1:5
        fprintf('\tMode %g:\t\t%.2f Hz\n',i,sqrt(values(i))/(2*pi));
    end
    fprintf('\n');
    
    % effective stiffness
    disp('Effective stiffness:');
    for i = 1:4
        fprintf('\tH%g:\t\t%.2f\n',i,Props.hEffectiveStiffness(i));
    end
    fprintf('\n');
    for i = 1:4
        fprintf('\tUV%g:\t%.2f\n',i,Props.uvEffectiveStiffness(i));
    end
    fprintf('\n');
    for i = 1:4
        fprintf('\tLV%g:\t%.2f\n',i,Props.lvEffectiveStiffness(i));
    end
    fprintf('\n');
    
    % estimated wire lengths
    disp('Estimated wire lengths:');
    fprintf('\tH:\t\t%.3f m\n',Props.hWireLength);
    fprintf('\tUV:\t\t%.3f m\n',Props.uvWireLength);
    fprintf('\tLV:\t\t%.3f m\n',Props.lvWireLength);
    fprintf('\n');
    
    % estimated pretension lengths
    disp('Estimated pretension lengths:');
    for i = 1:4
        fprintf('\tUV%g:\t%.3f m\n',i,(Props.initialForce(i+2)+Props.suspendedMass*9.81/4-Props.uvConstantSpringForce(i))/Props.uvEffectiveStiffness(i));
    end
    fprintf('\n');
    for i = 1:4
        fprintf('\tLV%g:\t%.3f m\n',i,(Props.initialForce(i+2)-Props.lvConstantSpringForce(i))/Props.lvEffectiveStiffness(i));
    end
    fprintf('\n\tH1: \t%.3f m\n',(Props.initialForce(1)-Props.hConstantSpringForce(1))/Props.hEffectiveStiffness(1));
    fprintf('\tH2: \t%.3f m\n',(Props.initialForce(1)-Props.hConstantSpringForce(2))/Props.hEffectiveStiffness(2));
    fprintf('\tH3: \t%.3f m\n',(Props.initialForce(2)-Props.hConstantSpringForce(3))/Props.hEffectiveStiffness(3));
    fprintf('\tH4: \t%.3f m\n\n',(Props.initialForce(2)-Props.hConstantSpringForce(4))/Props.hEffectiveStiffness(4));
end

end