function [displacements,fs] = loaddata(filename,startTime,endTime,Props)
%   LOADDATA Loads measured data and transform it into displacements of system DOFs.
%
%   INPUT
%       filename
%           Input text file containing measured spring forces. Columns:
%               1. Time
%               5. H1
%               2. H4
%               3. LV1
%               4. LV2
%               6. LV3
%               7. LV4
%       startTime
%           Start position of the time window to be loaded, given in
%           seconds.
%       endTime
%           End position of the time window to be loaded, given in
%           seconds. If the position is beyond the end of the time series,
%           the window will end there too.
%       Props
%           Structure containing all properties of the rig.
%           See also getrigproperties.
%
%   OUPUT
%       Displacement of system DOFs. Columns:
%           1. Horizontal translation
%           1. Horizontal rotation
%           1. Vertical translation
%           1. Vertical rotation
%           1. Longitudinal (torsional) rotation

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% VALIDATE INPUT
% ... 0 <= startPos < endPos

%% LOAD AND PROCESS R-DATA
fid = fopen([filename '.txt'],'r');
str = strrep(fread(fid,'*char')',',','.');                  % convert commas to dots
data = textscan(str,'','delimiter','\t','collectoutput',true);
clear str;
fclose(fid);

% extract data
fs = 1/(data{1}(2,1)-data{1}(1,1));
startPos = floor(startTime*fs+1);
endPos = ceil(endTime*fs+1);
if endPos > size(data{1},1)
    endPos = size(data{1},1);
end
force = data{1}(startPos:endPos,2:end);
clear data;

% remove constant spring forces
initialForce = Props.initialForce;
initialForce(1) = initialForce(1)-Props.hConstantSpringForce(1);
initialForce(2) = initialForce(2)-Props.hConstantSpringForce(4);
force(:,1) = force(:,1)-Props.hConstantSpringForce(1);
force(:,2) = force(:,2)-Props.hConstantSpringForce(4);
for i = 3:6
    initialForce(i) = initialForce(i)-Props.lvConstantSpringForce(i-2);
    force(:,i) = force(:,i)-Props.lvConstantSpringForce(i-2);
end

% estimate spring lengths
springLength = zeros(size(force));
springLength(:,1) = force(:,1)/Props.hSpringStiffness(1)+Props.hSpringLength;
springLength(:,2) = force(:,2)/Props.hSpringStiffness(4)+Props.hSpringLength;
for i = 3:6
    springLength(:,i) = force(:,i)/Props.lvSpringStiffness(i-2)+Props.lvSpringLength;
end

% estimate horizontal displacements
displacements = zeros(size(force));
displacements(:,1) = springLength(:,1)-initialForce(1)/Props.hSpringStiffness(1)-Props.hSpringLength;
displacements(:,2) = springLength(:,2)-initialForce(2)/Props.hSpringStiffness(4)-Props.hSpringLength;

% estimate vertical displacements (corrected with respect to estimated horizontal displacements)
for i = 3:6
    side = round(i/3);
    initialSuspensionLength = Props.lvWireLength+Props.lvSpringLength+initialForce(i)/Props.lvSpringStiffness(i-2);
    displacements(:,i) = sqrt((Props.lvWireLength+springLength(:,i)).^2-displacements(:,side).^2)-initialSuspensionLength;
end

% correct horizontal displacements (with respect to estimated vertical displacements)
displacements(:,1) = sqrt((Props.hWireLength+springLength(:,1)).^2-displacements(:,3).^2)-(Props.hWireLength+Props.hSpringLength+initialForce(1)/Props.hSpringStiffness(1));
displacements(:,2) = sqrt((Props.hWireLength+springLength(:,2)).^2-displacements(:,6).^2)-(Props.hWireLength+Props.hSpringLength+initialForce(2)/Props.hSpringStiffness(4));

% transform into displacements of system dofs
T = [
    0.5    -0.5     0       0       0
    0.5     0.5     0       0       0
    0       0       0.25    0.25    0.25
    0       0       0.25    0.25   -0.25
    0       0       0.25   -0.25   -0.25
    0       0       0.25   -0.25    0.25
    ];
T(:,2) = T(:,2)/Props.distanceCentreOfMassToHSprings;
T(:,4) = T(:,4)/Props.distanceCentreOfMassToVSprings;
T(:,5) = T(:,5)/Props.distanceLongitudinalAxisToVSprings;
displacements = displacements*T;

end