function fig = plotmatrixcomparison(reference,varargin)
%   PLOTMATRIXCOMPARISON Plots a comparison of one or more matrices versus
%   a reference matrix.
%
%   REQUIRED INPUT
%       [reference]
%           Reference matrix.
%       'title',[matrix]
%           One or more matrices to be compared to each other and the
%           reference matrix, given as title/matrix pairs.
%
%   OPTIONAL PARAMETERS
%       Key                       | Value
%       -------------------------------------------------------------------
%       'figName'                 | Figure name.
%       'includeTitle'            | Whether or not to include the title
%                                 |     above each matrix.
%       'plotStructure'           | Vector containing two elements: number
%                                 |     of rows and number of columns, respectively.
%       'elementSize'             | Size of each square representing an
%                                 |     element in the matrix. Given in
%                                 |     centimeters.
%       'verticalGap'             | Characteristic gap (vertical direction)
%                                 |     between entities. Given in centimeters.
%       'horizontalGap'           | Characteristic gap (horizontal direction)
%                                 |     between entities. Given in centimeters.
%       'radiusReductionScale'    | Scaling of circle radius. 1 renders a
%                                 |     circle that fill the element square.
%       'colorPositiveReference'  | Color of positive reference circles.
%       'colorNegativeReference'  | Color of negative reference circles.
%       'colorPositiveValue'      | Color of positive (filled) circles.
%       'colorNegativeValue'      | Color of negative (filled) circles.
%       'includeLegend'           | Whether or not to include a legend.
%       'legendPositiveReference' | Legend label for positive reference values.
%       'legendNegativeReference' | Legend label for negative reference values.
%       'legendPositiveValue'     | Legend label for positive values.
%       'legendNegativeValue'     | Legend label for negative values.
%
%   OUPUT
%       fig
%           Figure handle.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% SOFTWARE CHECK
recommendedMatlabVersion = 7.14;
v = ver('matlab');
if isempty(v)
    error('IDENTIFICATION:missingSoftware','Missing software: MATLAB');
elseif str2double(v.Version) < recommendedMatlabVersion
    warning('IDENTIFICATION:softwareVersion',...
        'The installed version of MATLAB (%s) is older than the version used during developement of this function (%g), and hence it may not work properly.',...
        v.Version,recommendedMatlabVersion);
end

%% DEFAULT SETTINGS
figName = 'Matrix comparison';
includeTitle = true;
plotStructure = [1 inf];
elementSize = 2;
verticalGap = 0.4;
horizontalGap = 0.8;
radiusReductionScale = 0.9;
colorPositiveReference = 'k';
colorNegativeReference = 'k';
colorPositiveValue = [0.45 0.65 1.00];
colorNegativeValue = [1.00 0.45 0.30];
includeLegend = true;
legendPositiveReference = 'Reference (positive values)';
legendNegativeReference = 'Reference (negative values)';
legendPositiveValue = 'Values (positive)';
legendNegativeValue = 'Values (negative)';

%% PARSE AND VALIDATE INPUT
% validation functions
isbool = @(x) islogical(x) && isscalar(x);
isrealscalar = @(x) ~ischar(x) && isreal(x) && isscalar(x);
isnonnegativescalar = @(x) isrealscalar(x) && x > 0;
isrealmatrix = @(x) ismatrix(x) && isnumeric(x) && isreal(x);
isplotstructure = @(x) isvector(x) && numel(x) == 2 && all(arrayfun(isnonnegativescalar,x));
iscolor = @(x) (ischar(x) && length(x) <= 2) || (isnumeric(x) && isvector(x) && numel(x) == 3 && min(x) >= 0 && max(x) <= 1);

% build input scheme
p = inputParser;
p.FunctionName = 'plotmatrixcomparison';
p.CaseSensitive = false;
p.KeepUnmatched = true;
p.addRequired('reference',isrealmatrix);
p.addParamValue('figName',figName,@ischar);
p.addParamValue('includeTitle',includeTitle,isbool);
p.addParamValue('plotStructure',plotStructure,isplotstructure);
p.addParamValue('elementSize',elementSize,isnonnegativescalar);
p.addParamValue('verticalGap',verticalGap,isnonnegativescalar);
p.addParamValue('horizontalGap',horizontalGap,isnonnegativescalar);
p.addParamValue('radiusReductionScale',radiusReductionScale,isnonnegativescalar);
p.addParamValue('colorPositiveReference',colorPositiveReference,iscolor);
p.addParamValue('colorNegativeReference',colorNegativeReference,iscolor);
p.addParamValue('colorPositiveValue',colorPositiveValue,iscolor);
p.addParamValue('colorNegativeValue',colorNegativeValue,iscolor);
p.addParamValue('includeLegend',includeLegend,isbool);
p.addParamValue('legendPositiveReference',legendPositiveReference,@ischar);
p.addParamValue('legendNegativeReference',legendNegativeReference,@ischar);
p.addParamValue('legendPositiveValue',legendPositiveValue,@ischar);
p.addParamValue('legendNegativeValue',legendNegativeValue,@ischar);
p.parse(reference,varargin{:});

% get parameters
figName = p.Results.figName;
includeTitle = p.Results.includeTitle;
plotStructure = p.Results.plotStructure;
elementSize = p.Results.elementSize;
verticalGap = p.Results.verticalGap;
horizontalGap = p.Results.horizontalGap;
radiusReductionScale = p.Results.radiusReductionScale;
colorPositiveReference = p.Results.colorPositiveReference;
colorNegativeReference = p.Results.colorNegativeReference;
colorPositiveValue = p.Results.colorPositiveValue;
colorNegativeValue = p.Results.colorNegativeValue;
includeLegend = p.Results.includeLegend;
legendPositiveReference = p.Results.legendPositiveReference;
legendNegativeReference = p.Results.legendNegativeReference;
legendPositiveValue = p.Results.legendPositiveValue;
legendNegativeValue = p.Results.legendNegativeValue;

% validate parameters
if radiusReductionScale > 1
    error('IDENTIFICATION:PLOT_MATRIX_COMPARISON:radiusReductionScale','The radius reduction scale (%g) has to be within the interval [0,1].',radiusReductionScale);
end

% get matrices (sorted by argument order)
matrixTitle = fieldnames(p.Unmatched);
nMatrices = numel(matrixTitle);
order = zeros(nMatrices,1);
for i = 1:nMatrices
    for j = 1:2:numel(varargin)
        if ischar(varargin{j}) && strcmpi(varargin{j},matrixTitle{i})
            if order(i) > 0
                error('IDENTIFICATION:PLOT_MATRIX_COMPARISON:equalTitles','Two er more matrices have the same title.');
            end
            order(i) = j;
        end
    end
end
[~,ind] = sort(order,'ascend');
matrixTitle = matrixTitle(ind);
matrix = cell(nMatrices,1);
for i = 1:nMatrices
    if ~isrealmatrix(p.Unmatched.(matrixTitle{i}))
        if ~isreal(p.Unmatched.(matrixTitle{i}))
            error('IDENTIFICATION:PLOT_MATRIX_COMPARISON:complexMatrix','The matrix "%s" contains complex values.',matrixTitle{i});
        else
            error('IDENTIFICATION:PLOT_MATRIX_COMPARISON:invalidMatrix','The matrix "%s" is not valid.',matrixTitle{i});
        end
    end
    matrix{i} = p.Unmatched.(matrixTitle{i});
end

% validate matrices
if nMatrices == 0
    error('IDENTIFICATION:PLOT_MATRIX_COMPARISON:missingMatrix','At least one matrix has to be specified (in addition to the reference matrix).');
end
if isinf(max(plotStructure)) && isinf(min(plotStructure))
    error('IDENTIFICATION:PLOT_MATRIX_COMPARISON:infPlotStructure','The plot structure is ambiguous, since both the the number of rows and columns are set to "inf".');
elseif ~isinf(max(plotStructure)) && plotStructure(1)*plotStructure(2) < nMatrices
    error('IDENTIFICATION:PLOT_MATRIX_COMPARISON:smallPlotStructure','The plot structure is too small to contain all matrices.');
end
[nElements1,nElements2] = size(reference);
for i = 1:nMatrices
    [nRows,nCols] = size(matrix{i});
    if nRows ~= nElements1
        error('IDENTIFICATION:PLOT_MATRIX_COMPARISON:rows','"%s" does not have the same number of rows as the reference matrix.',matrixTitle{i});
    elseif nCols ~= nElements2
        error('IDENTIFICATION:PLOT_MATRIX_COMPARISON:cols','"%s" does not have the same number of columns as the reference matrix.',matrixTitle{i});
    end
end

%% MAXIMUM AND MINIMUM VALUES
maxReference = max(max(reference));
minReference = min(min(reference));
maxValue = -inf;
minValue = inf;
for i = 1:nMatrices
    maxValue = max(maxValue,max(max(matrix{i})));
    minValue = min(minValue,min(min(matrix{i})));
end
absMax = max(abs(max(maxReference,maxValue)),abs(min(minReference,minValue)));

%% PLOT
units = 'centimeters';
fig = figure('name',figName,'units',units,'visible','off');
ax = cell(nMatrices,1);
n = max(nElements1,nElements2);
for k = 1:nMatrices
    ax{k} = axes('units',units);
    axis equal;
    hold on;
    if includeTitle
        title(matrixTitle{k});
    end
    set(ax{k},'xlim',[0 nElements2/n],'ylim',[0 nElements1/n]);
    set(ax{k},'xtick',[],'ytick',[]);
    set(ax{k},'xticklabel',{},'yticklabel',{});
    
    % values (proportinal to circle radius)
    for i = 1:nElements1
        for j = 1:nElements2
            r_val = matrix{k}(i,j)/absMax;
            [x,y] = circle(i,j,r_val);
            if r_val > 0
                fill(x,y,colorPositiveValue,'edgecolor',colorPositiveValue);
            else
                fill(x,y,colorNegativeValue,'edgecolor',colorNegativeValue);
            end
            r_ref = reference(i,j)/absMax;
            [x,y] = circle(i,j,r_ref);
            if r_ref > 0
                plot(x,y,'color',colorPositiveReference,'linewidth',2);
            else
                plot(x,y,':','color',colorNegativeReference,'linewidth',2);
            end
        end
    end
    
    % grid
    for i = 1:(nElements1-1)
        plot([0 1],[i/n i/n],'color',0.6*[1 1 1]);
    end
    for i = 1:(nElements2-1)
        plot([i/n i/n],[0 1],'color',0.6*[1 1 1]);
    end
    
    % frame
    plot([0 0 nElements2/n nElements2/n 0],[0 nElements1/n nElements1/n 0 0],'k');
end

% legend
axes('units',units,'visible','off');
hold on;
labels = {};

if maxReference >= 0 && abs(maxReference) > 0.01*abs(minReference)
    plot([0 0],[0 0],'color',colorPositiveReference,'linewidth',2,'visible','off');
    labels = [labels legendPositiveReference];
end
if minReference < 0 && abs(minReference) > 0.01*abs(maxReference)
    plot([0 0],[0 0],':','color',colorNegativeReference,'linewidth',2,'visible','off');
    labels = [labels legendNegativeReference];
end
if maxValue >= 0
    fill([0 0],[0 0],colorPositiveValue,'visible','off');
    labels = [labels legendPositiveValue];
end
if minValue < 0
    fill([0 0],[0 0],colorNegativeValue,'visible','off');
    labels = [labels legendNegativeValue];
end
if includeLegend
    l = legend(labels,'orientation','horizontal','units',units);
    lPos = get(l,'position');
end

%% RESIZE AND REPOSITION ELEMENTS
nPlotRows = plotStructure(1);
nPlotCols = plotStructure(2);
if nPlotRows > nMatrices
    nPlotRows = nMatrices;
end
if isinf(nPlotCols)
    nPlotCols = ceil(nMatrices/nPlotRows);
end
if nPlotCols > nMatrices
    nPlotCols = nMatrices;
end
nPlotRows = ceil(nMatrices/nPlotCols);
plotH = elementSize*nElements1;
plotW = elementSize*nElements2;
if includeLegend
    bottom = lPos(4)+verticalGap;
else
    bottom = 0;
end
for i = nPlotRows:-1:1
    bottom = bottom+verticalGap;
    titleHeight = 0;
    c = nPlotCols;
    if i == nPlotRows
        c = nPlotCols-(nPlotCols*nPlotRows-nMatrices);
    end
    for j = 1:c
        set(ax{(i-1)*nPlotCols+j},'position',[(j-1)*plotW+j*horizontalGap,bottom,plotW,plotH]);
        tmp = get(ax{i},'tightinset');
        titleHeight = max(titleHeight,tmp(4));
    end
    bottom = bottom+plotH+titleHeight+verticalGap;
end
figW = nPlotCols*plotW+(nPlotCols+1)*horizontalGap;
figH = bottom;
if includeLegend
    set(l,'position',[(figW-lPos(3))/2,verticalGap,lPos(3),lPos(4)]);
end
set(fig,'position',[0,0,figW,figH]);

% show figure
movegui(fig,'center');
set(fig,'visible','on');

%% AUXILIARY PLOT FUNCTION
    function [x,y] = circle(i,j,radius)
        r = abs(radius)*0.5/n*radiusReductionScale;
        angle = linspace(0,2*pi,round((1+100*sqrt(r))*10));
        x = cos(angle)*r+(j-0.5)/n;
        y = sin(angle)*r-(i-nElements1-0.5)/n;
    end

end