function fig = plotsignal(dt,varargin)
%   PLOTSIGNAL Plots one or more sets of an n-channel signal in time and/or
%   frequency domain.
%
%   REQUIRED INPUT
%       dt
%           Sampling interval.
%       'signalSetTitle',[signalSet]
%           One or more signal set time series, given as title/set pairs.
%           A signal set is a matrix where each column represents a channel.
%
%   OPTIONAL PARAMETERS
%       Key                 | Value
%       -------------------------------------------------------------------
%       'figName'           | Figure name.
%       'includeTimeSeries' | Whether or not to include time series plots.
%       'includePsd'        | Whether or not to include PSD plots.
%       'includeLegend'     | Whether or not to include a legend.
%       'channelTitle'      | Cell array containing the channel titles.
%       'channelYLabel'     | Cell array containing the labels on the
%                           |     Y-axis of each channel.
%       'tLabel'            | Label on the time axis.
%       't0'                | First value on the time axis.
%       'cutoffFrequency'   | Maximum value on the frequency axis.
%       'plotWidth'         | Width of each time series and/or PSD plot.
%                           |     Given in centimeters.
%       'plotHeight'        | Height of each time series and/or PSD plot.
%                           |     Given in centimeters.
%       'verticalGap'       | Characteristic gap (vertical direction)
%                           |     between entities. Given in centimeters.
%       'horizontalGap'     | Characteristic gap (horizontal direction)
%                           |     between entities. Given in centimeters.
%       'titleOffset'       | Title label offset from corresponding plot.
%       'precision'         | Number of digits to the right of the decimal
%                           | point. Can be provided as a single value.
%                           | affecting all channels, or as a vector with a
%                           | specified precision for each channel.
%       'equalYAxes'        | Whether or not to use the same Y-axes for all
%                           | channels.
%       'optimizeYAxes'     | Whether or not to optimize ticks on the Y-axes.
%       'equalMaxMin'       | Whether or not to make absolute values of
%                           | maximum and minimum values equal on the Y-axes.
%       'nYTicks'           | Number of ticks on the Y-axes.
%
%   OUPUT
%       fig
%           Figure handle.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% SOFTWARE CHECK
recommendedMatlabVersion = 7.14;
v = ver('matlab');
if isempty(v)
    error('IDENTIFICATION:missingSoftware','Missing software: MATLAB');
elseif str2double(v.Version) < recommendedMatlabVersion
    warning('IDENTIFICATION:softwareVersion',...
        'The installed version of MATLAB (%s) is older than the version used during developement of this function (%g), and hence it may not work properly.',...
        v.Version,recommendedMatlabVersion);
end

%% DEFAULT SETTINGS
figName = 'Signals';
includeTimeSeries = true;
includePsd = true;
includeLegend = true;
channelTitle = {};
channelYLabel = {};
tLabel = 'Time (s)';
t0 = 0;
cutoffFrequency = inf;
plotWidth = 10;
plotHeight = 2;
verticalGap = 0.4;
horizontalGap = 0.8;
titleOffset = NaN;
precision = inf;
equalYAxes = false;
optimizeYAxes = false;
equalMaxMin = false;
nYTicks = NaN;

% line style
defaultStyle = '-';
defaultThickness = 1;
defaultColorMap = @(N) hsv(N);

%% PARSE AND VALIDATE INPUT
% validation functions
isbool = @(x) islogical(x) && isscalar(x);
isrealscalar = @(x) ~ischar(x) && isreal(x) && isscalar(x);
isnonnegativescalar0 = @(x) isrealscalar(x) && x >= 0;
isnonnegativescalar = @(x) isrealscalar(x) && x > 0;
isnonnegativevector0 = @(x) isvector(x) && all(arrayfun(isnonnegativescalar0,x));
isnonnegativevector = @(x) isvector(x) && all(arrayfun(isnonnegativescalar,x));
isrealmatrix = @(x) ismatrix(x) && isnumeric(x) && isreal(x);
ischarcell = @(x) iscell(x) && all(cellfun(@ischar,x));
iscolor = @(x) (ischar(x) && length(x) <= 2) || (isnumeric(x) && isvector(x) && numel(x) == 3 && min(x) >= 0 && max(x) <= 1);

% build input scheme
p = inputParser;
p.FunctionName = 'plotsignal';
p.CaseSensitive = false;
p.KeepUnmatched = true;
p.addRequired('dt',isnonnegativescalar);
p.addParamValue('figName',figName,@ischar);
p.addParamValue('includeTimeSeries',includeTimeSeries,isbool);
p.addParamValue('includePsd',includePsd,isbool);
p.addParamValue('includeLegend',includeLegend,isbool);
p.addParamValue('channelTitle',channelTitle,ischarcell);
p.addParamValue('channelYLabel',channelYLabel,ischarcell);
p.addParamValue('tLabel',tLabel,@ischar);
p.addParamValue('t0',t0,isrealscalar);
p.addParamValue('cutoffFrequency',cutoffFrequency,isnonnegativevector);
p.addParamValue('plotWidth',plotWidth,isnonnegativescalar);
p.addParamValue('plotHeight',plotHeight,isnonnegativescalar);
p.addParamValue('verticalGap',verticalGap,isnonnegativescalar0);
p.addParamValue('horizontalGap',horizontalGap,isnonnegativescalar0);
p.addParamValue('titleOffset',titleOffset,isnonnegativescalar0);
p.addParamValue('precision',precision,isnonnegativevector0);
p.addParamValue('equalYAxes',equalYAxes,isbool);
p.addParamValue('optimizeYAxes',optimizeYAxes,isbool);
p.addParamValue('equalMaxMin',equalMaxMin,isbool);
p.addParamValue('nYTicks',nYTicks,isnonnegativescalar);
p.parse(dt,varargin{:});

% get parameters
figName = p.Results.figName;
includeTimeSeries = p.Results.includeTimeSeries;
includePsd = p.Results.includePsd;
includeLegend = p.Results.includeLegend;
channelTitle = p.Results.channelTitle;
channelYLabel = p.Results.channelYLabel;
tLabel = p.Results.tLabel;
t0 = p.Results.t0;
cutoffFrequency = p.Results.cutoffFrequency;
plotWidth = p.Results.plotWidth;
plotHeight = p.Results.plotHeight;
verticalGap = p.Results.verticalGap;
horizontalGap = p.Results.horizontalGap;
titleOffset = p.Results.titleOffset;
precision = p.Results.precision;
equalYAxes = p.Results.equalYAxes;
optimizeYAxes = p.Results.optimizeYAxes;
equalMaxMin = p.Results.equalMaxMin;
nYTicks = p.Results.nYTicks;

% validate parameters and set (default) values based on given arguments
fs = 1/dt;                                                  % average sampling frequency
nyq = fs/2;                                                 % nyquist frequency
if ~includeTimeSeries && ~includePsd
    warning('IDENTIFICATION:PLOT_SIGNAL:nothingToPlot','No plot will be made, since neither time series or PSD are included.');
    return;
end
if isnan(titleOffset)
    titleOffset = plotHeight/10;                            % default value
end
if isnan(nYTicks)
    nYTicks = round(plotHeight*2);                          % default value
elseif nYTicks < 2
    warning('IDENTIFICATION:PLOT_SIGNAL:nYTicks','Number of y-ticks (%g) is too low, and is set to the minimum value (2).',nYTicks);
    nYTicks = 2;
end

% get signal sets (sorted by argument order)
signalSetTitle = fieldnames(p.Unmatched);
signalSetTitle(...
    strcmpi(signalSetTitle,'style')|...
    strcmpi(signalSetTitle,'color')|...
    strcmpi(signalSetTitle,'thickness')...
    ) = [];
nSignalSets = numel(signalSetTitle);
order = zeros(nSignalSets,1);
for i = 1:nSignalSets
    for j = 1:2:numel(varargin)
        if ischar(varargin{j}) && strcmpi(varargin{j},signalSetTitle{i})
            if order(i) > 0
                error('IDENTIFICATION:PLOT_SIGNAL:equalTitles','Two er more signal sets have the same title.');
            end
            order(i) = j;
        end
    end
end
[pos,ind] = sort(order,'ascend');
signalSetTitle = signalSetTitle(ind);
signalSet = cell(nSignalSets,1);
for i = 1:nSignalSets
    if ~isrealmatrix(p.Unmatched.(signalSetTitle{i}))
        error('IDENTIFICATION:PLOT_SIGNAL:invalidSignalSet','The signal set "%s" is not valid.',signalSetTitle{i});
    end
    signalSet{i} = varargin{pos(i)+1};
end

% validate signal sets
if nSignalSets == 0
    error('IDENTIFICATION:PLOT_SIGNAL:missingMatrix','At least one signal set has to be specified.');
end
[nSamples,nChannels] = size(signalSet{1});
for i = 2:nSignalSets
    [nRows,nCols] = size(signalSet{i});
    if nRows ~= nSamples
        error('IDENTIFICATION:PLOT_SIGNAL:samples','"%s" does not have the same number of samples (%g) as the first signal set (%g samples).',signalSetTitle{i},nRows,nSamples);
    elseif nCols ~= nChannels
        error('IDENTIFICATION:PLOT_SIGNAL:channels','"%s" does not have the same number of channels (%g) as the first signal set (%g channels).',signalSetTitle{i},nCols,nChannels);
    end
end

% validate channel properties
if ~isempty(channelTitle) && numel(channelTitle) ~= nChannels
    error('IDENTIFICATION:PLOT_SIGNAL:channelNames','The number of channel names (%g) and channels (%g) does not match.',numel(channelTitle),nChannels);
end
if ~isempty(channelYLabel) && numel(channelYLabel) ~= nChannels
    error('IDENTIFICATION:PLOT_SIGNAL:channelYLabels','The number of channel y-labels (%g) and channels (%g) does not match.',numel(channelYLabel),nChannels);
end
if isscalar(cutoffFrequency)
    if cutoffFrequency > nyq
        if ~isinf(cutoffFrequency)
            warning('IDENTIFICATION:PLOT_SIGNAL:cutoffNyquistFreq',...
                'The cut-off frequency (%.2f Hz) is higher than the Nyquist frequency (%.2f Hz), and will be reduced to this value.',...
                cutoffFrequency,nyq);
        end
        cutoffFrequency = nyq;
    end
    cutoffFrequency = repmat(cutoffFrequency,nChannels,1);
else
    if numel(cutoffFrequency) ~= nChannels
        error('IDENTIFICATION:PLOT_SIGNAL:channelCutoffFrequency',...
            'The number of channel cut-off-frequencies (%g) and channels (%g) does not match. If you want to specify a single cut-off-frequency for all channels, provide a single value.',...
            numel(cutoffFrequency),nChannels);
    end
    for i = 1:nChannels
        if cutoffFrequency(i) > nyq
            if ~isinf(cutoffFrequency(i))
                warning('IDENTIFICATION:PLOT_SIGNAL:cutoffNyquistFreq',...
                    'The cut-off frequency given for channel %g (%.2f Hz) is higher than the Nyquist frequency (%.2f Hz), and will be reduced to this value.',...
                    i,cutoffFrequency(i),nyq);
            end
            cutoffFrequency(i) = nyq;
        end
    end
end
if isscalar(precision)
    precision = repmat(precision,nChannels,1);
elseif numel(precision) ~= nChannels
    error('IDENTIFICATION:PLOT_SIGNAL:channelPrecision',...
        'The number of channel precision values (%g) and channels (%g) does not match. If you want to specify a single precision value for all channels, provide a single value.',...
        numel(precision),nChannels);
end

% get and validate signal set properties
style = repmat({defaultStyle},nSignalSets,1);
color = mat2cell(defaultColorMap(nSignalSets),ones(nSignalSets,1),3);
thickness = ones(nSignalSets,1)*defaultThickness;
for i = 1:nSignalSets
    if i ~= nSignalSets
        b = pos(i+1)-2;
    else
        b = numel(varargin)-1;
    end
    for p = pos(i)+2:b
        if strcmpi(varargin{p},'style')
            if ~ischar(varargin{p+1}) || length(varargin{p+1}) > 2
                error('IDENTIFICATION:PLOT_SIGNAL:invalidStyle','Invalid line style specified for "%s".',signalSetTitle{i});
            end
            style{i} = varargin{p+1};
        elseif strcmpi(varargin{p},'color')
            if ~iscolor(varargin{p+1})
                error('IDENTIFICATION:PLOT_SIGNAL:invalidColor','Invalid line color specified for "%s".',signalSetTitle{i});
            end
            color{i} = varargin{p+1};
        elseif strcmpi(varargin{p},'thickness')
            if ~isnonnegativescalar(varargin{p+1}) || varargin{p+1} < 1
                error('IDENTIFICATION:PLOT_SIGNAL:invalidThickness','Invalid line thickness specified for "%s".',signalSetTitle{i});
            end
            thickness(i) = varargin{p+1};
        end
    end
end

%% PLOT
units = 'centimeters';
fig = figure('name',figName,'units',units,'visible','off');
if includeTimeSeries && includePsd
    nPlotCols = 2;
else
    nPlotCols = 1;
end
ax = cell(nSignalSets,nPlotCols);
axTitle = cell(nChannels,1);
t = t0+(0:dt:dt*(nSamples-1));
maxAllY = ones(nPlotCols,1)*-inf;
minAllY = ones(nPlotCols,1)*inf;

% plot properties
for c = 1:nChannels
    % time series
    plotCol = 1;
    if includeTimeSeries
        ax{c,plotCol} = axes('units',units,'position',[0,0,plotWidth,plotHeight]);
        hold on;
        maxY = -inf;
        minY = inf;
        for i = 1:nSignalSets
            plot(t,signalSet{i}(:,c),'linestyle',style{i},'linewidth',thickness(i),'color',color{i});
            maxY = max(maxY,max(signalSet{i}(:,c)));
            minY = min(minY,min(signalSet{i}(:,c)));
        end
        xlim([t(1) t(end)]);
        xlabel(tLabel);
        if ~isempty(channelYLabel)
            ylabel(channelYLabel{c});
        else
            ylabel('Signal');
        end
        
        % set y-limit
        [minY,maxY] = getminmax(minY,maxY);
        maxAllY(plotCol) = max(maxAllY(plotCol),maxY);
        minAllY(plotCol) = min(minAllY(plotCol),minY);
        setylim(gca,c,minY,maxY,true);
    end
    
    % psd
    if includePsd
        if includeTimeSeries
            plotCol = 2;
        end
        ax{c,plotCol} = axes('units',units,'position',[0,0,plotWidth,plotHeight]);
        hold on;
        maxY = -inf;
        minY = inf;
        for i = 1:nSignalSets
            % fourier transform
            nFft = 2^nextpow2(nSamples);                    % power of 2 speeds up fft
            fftSeries = fft(signalSet{i}(:,c),nFft);        % fast fourier transform
            nUniquePoints = ceil((nFft+1)/2);               % number of unique points
            fftSeries = fftSeries(1:nUniquePoints,:);       % fft is symmetric, discard second half
            power = abs(fftSeries).^2/nSamples;             % scaled power of fft
            
            % since half of fft has been discarded, amplitudes must be multiplied by 2.
            % the dc component and nyquist component are unique and should not be multiplied by 2.
            power(2:end-1) = power(2:end-1)*2;
            
            % frequency range
            f = linspace(0,nyq,nUniquePoints);
            nPoints = ceil((nUniquePoints-1)*cutoffFrequency(c)/nyq)+1;
            f = f(1:nPoints);
            power = power(1:nPoints);
            
            % plot
            plot(f,power,'linestyle',style{i},'linewidth',thickness(i),'color',color{i});
            maxY = max(maxY,max(power));
            minY = min(minY,min(power));
        end
        xlim([0 f(nPoints)]);
        xlabel('Frequency (Hz)');
        ylabel('PSD');
        
        % set y-limit
        [minY,maxY] = getminmax(minY,maxY);
        maxAllY(plotCol) = max(maxAllY(plotCol),maxY);
        minAllY(plotCol) = min(minAllY(plotCol),minY);
        setylim(gca,c,minY,maxY,false);
    end
    
    % channel names
    if ~isempty(channelTitle)
        axTitle{c} = axes('units',units,'visible','off');
        title(channelTitle{c},'fontweight','bold','visible','on');
    end
end

% equal y-axis
if equalYAxes
    for c = 1:nChannels
        for i = 1:nPlotCols
            setylim(ax{c,i},c,minAllY(i),maxAllY(i),i == 1 && includeTimeSeries);
        end
    end
end

% frame
for c = 1:nChannels
    for i = 1:nPlotCols
        xl = xlim(ax{c,i});
        yl = ylim(ax{c,i});
        plot(ax{c,i},[xl(1) xl(1)],[yl(1) yl(2)],'k',[xl(1) xl(2)],[yl(1) yl(1)],'k');
    end
end

% legend
lPos = zeros(4,1);
if includeLegend
    axes('units',units,'visible','off');
    hold on;
    for i = nSignalSets:-1:1
        plot([0 0],[0 0],'linestyle',style{i},'linewidth',thickness(i),'color',color{i},'visible','off');
    end
    l = legend(flipud(signalSetTitle),'orientation','horizontal','units',units);
    lPos = get(l,'position');
end

%% REPOSITION ELEMENTS
% calculate necessary tight insets
lInset = zeros(nPlotCols,1);
rInset = zeros(nPlotCols,1);
bInset = zeros(nChannels,1);
tInset = zeros(nChannels,1);
titleHeight = zeros(nChannels,1);
for c = 1:nChannels
    for i = 1:nPlotCols
        inset = get(ax{c,i},'tightinset');
        inset(4) = titleOffset;
        if inset(1) > lInset(i)
            lInset(i) = inset(1);
        end
        if inset(3) > rInset(i)
            rInset(i) = inset(3);
        end
        if inset(2) > bInset(c)
            bInset(c) = inset(2);
        end
        if inset(4) > tInset(c)
            tInset(c) = inset(4);
        end
    end
    if ~isempty(channelTitle)
        inset = get(axTitle{c},'tightinset');
        titleHeight(c) = inset(4);
        if nPlotCols > 1
            bInset(c) = 2/3*bInset(c);                    % unnecessary to use full spacing if channel names are displayed
        end
    end
end
if isempty(channelTitle) && nPlotCols > 1
    bInset(end) = bInset(end)/2;                          % inset of bottom plots is halved, regardless of the existance of channel names
end

% reposition
if includeLegend
    bottom = lPos(4)+2*verticalGap+bInset(end);
else
    bottom = verticalGap+3/2*bInset(end);
end
if nPlotCols > 1
    totalPlotW = 2*plotWidth+lInset(2)+rInset(1)+horizontalGap;
else
    totalPlotW = plotWidth;
end
for c = nChannels:-1:1
    set(ax{c,1},'position',[lInset(1)+horizontalGap,bottom,plotWidth,plotHeight]);
    if nPlotCols > 1
        set(ax{c,2},'position',[plotWidth+sum(lInset)+rInset(1)+2*horizontalGap,bottom,plotWidth,plotHeight]);
    end
    set(axTitle{c},'position',[lInset(1)+horizontalGap,bottom+tInset(c),totalPlotW,plotHeight]);
    bottom = bottom+plotHeight+tInset(c)+titleHeight(c)+verticalGap;
    if c > 1
        bottom = bottom+bInset(c-1);
    end
end
figW = nPlotCols*plotWidth+sum(lInset)+sum(rInset)+(nPlotCols+1)*horizontalGap;
figH = bottom;
set(fig,'position',[0,0,figW,figH]);
if includeLegend
    set(l,'position',[lInset(1)+horizontalGap+(totalPlotW-lPos(3))/2,verticalGap,lPos(3),lPos(4)]);
end

% show figure
movegui(fig,'center');
set(fig,'visible','on');

%% AUXILIARY FUNCTIONS
    function [newMin,newMax] = getminmax(currentMin,currentMax)
        if(optimizeYAxes)
            % round with respect to the first significant digit
            newMin = currentMin*1.05;
            newMax = currentMax*1.05;
            minLog = log10(abs(newMin));
            maxLog = log10(abs(newMax));
            factor = 10^(-floor(max(minLog,maxLog)));
            if ~isinf(minLog)                               % check if minimum is (approximately) zero by evaluating the logarithm
                newMin = floor(newMin*factor*2)/2/factor;
            else
                newMin = 0;
            end
            if ~isinf(maxLog)                               % check if maximum is (approximately) zero by evaluating the logarithm
                newMax = ceil(newMax*factor*2)/2/factor;
            else
                newMax = 0;
            end
        else
            yTick = get(gca,'ytick');
            newMin = yTick(1);
            newMax = yTick(end);
        end
        
        % avoid equal minimum and maximum
        if newMin == newMax
            newMin = newMin-3e-99;
            newMax = newMax+3e-99;
        end
    end

    function setylim(ax,channel,minY,maxY,isTimeSeries)
        % change limits
        if equalMaxMin && isTimeSeries
            if abs(minY) > abs(maxY)
                maxY = -minY;
            else
                minY = -maxY;
            end
        end
        ylim(ax,[minY maxY]);
        
        % change number of ticks
        if(optimizeYAxes)
            if ~isinf(nYTicks)
                y = linspace(minY,maxY,nYTicks);
                set(ax,'ytick',y);
            end
        end
        
        % format ticks
        if ~isinf(precision(channel))
            y = get(ax,'ytick')';
            absY = abs(y);
            numStr = cell(numel(y),1);
            for s = 1:numel(y)
                if y(s) == 0
                    numStr{s} = '0';
                    continue;
                end
                if absY(s) >= 10^4                          % switch to scientific notation if number is bigger than 9999
                    str = sprintf('%.*e',1,y(s));
                elseif absY(s) <= 10^(-precision(channel))  % switch to scientific notation if the number doesn't fit into the given precision
                    str = sprintf('%.*e',1,y(s));
                else                                        % otherwise, ensure correct precision
                    str = sprintf('%.*f',precision(channel),y(s));
                end
                numStr{s} = str;
            end
            set(ax,'yticklabel',numStr);
        end
    end

end