function fig = plotvectorcomparison(reference,varargin)
%   PLOTVECTORCOMPARISON Plots a comparison of one or more vectors versus
%   a reference vector.
%
%   REQUIRED INPUTS
%       [reference]
%           ...
%       'title',[vector]
%           ...
%
%   OPTIONAL PARAMETERS
%       Key               | Value
%       -------------------------------------------------------------------
%       'figName'         | Figure name.
%       'includeTitle'    | Whether or not to include the title
%                         |     above each matrix.
%       'minMaxValue'     | Smallest maximum value on the value axis.
%       'maxMinValue'     | Largest minimum value on the value axis.
%       'rangeScale'      | The maximum value on the value axis will be
%                         | determined by multiplying the largest plotted
%                         | value with this scale.
%       'plotSize'        | Width of plot, given in centimeters.
%       'verticalGap'     | Characteristic gap (vertical direction)
%                         |     between entities. Given in centimeters.
%       'horizontalGap'   | Characteristic gap (horizontal direction)
%                         |     between entities. Given in centimeters.
%       'valueLabel'      | Label on the value axis.
%       'includeLegend'   | Whether or not to include a legend.
%       'legendReference' | Legend label for reference values.
%       'legendValue'     | Legend label for values.
%
%   OUPUT
%       fig
%           Figure handle.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% SOFTWARE CHECK
recommendedMatlabVersion = 7.14;
v = ver('matlab');
if isempty(v)
    error('IDENTIFICATION:missingSoftware','Missing software: MATLAB');
elseif str2double(v.Version) < recommendedMatlabVersion
    warning('IDENTIFICATION:softwareVersion',...
        'The installed version of MATLAB (%s) is older than the version used during developement of this function (%g), and hence it may not work properly.',...
        v.Version,recommendedMatlabVersion);
end

%% DEFAULT SETTINGS
figName = 'Vector comparison';
includeTitle = true;
minMaxValue = NaN;
maxMinValue = NaN;
rangeScale = 1.05;
plotSize = 20;
verticalGap = 0.4;
horizontalGap = 0.8;
valueLabel = '';
includeLegend = true;
legendReference = 'Reference';
legendValue = 'Values';

%% PARSE AND VALIDATE INPUT
% validation functions
isbool = @(x) islogical(x) && isscalar(x);
isrealscalar = @(x) ~ischar(x) && isreal(x) && isscalar(x);
isnonnegativescalar = @(x) isrealscalar(x) && x > 0;
isrealvector = @(x) isvector(x) && isnumeric(x) && isreal(x);

% build input scheme
p = inputParser;
p.FunctionName = 'plotvectorcomparison';
p.CaseSensitive = false;
p.KeepUnmatched = true;
p.addRequired('reference',isrealvector);
p.addParamValue('figName',figName,@ischar);
p.addParamValue('includeTitle',includeTitle,isbool);
p.addParamValue('minMaxValue',minMaxValue,@isscalar);
p.addParamValue('maxMinValue',maxMinValue,@isscalar);
p.addParamValue('rangeScale',rangeScale,isnonnegativescalar);
p.addParamValue('plotSize',plotSize,isnonnegativescalar);
p.addParamValue('verticalGap',verticalGap,isnonnegativescalar);
p.addParamValue('horizontalGap',horizontalGap,isnonnegativescalar);
p.addParamValue('valueLabel',valueLabel,@ischar);
p.addParamValue('includeLegend',includeLegend,isbool);
p.addParamValue('legendValue',legendValue,@ischar);
p.addParamValue('legendReference',legendReference,@ischar);
p.parse(reference,varargin{:});

% get parameters
figName = p.Results.figName;
includeTitle = p.Results.includeTitle;
minMaxValue = p.Results.minMaxValue;
maxMinValue = p.Results.maxMinValue;
rangeScale = p.Results.rangeScale;
plotSize = p.Results.plotSize;
verticalGap = p.Results.verticalGap;
horizontalGap = p.Results.horizontalGap;
valueLabel = p.Results.valueLabel;
includeLegend = p.Results.includeLegend;
legendReference = p.Results.legendReference;
legendValue = p.Results.legendValue;

% get vectors (sorted by argument order)
vectorTitle = fieldnames(p.Unmatched);
nVectors = numel(vectorTitle);
order = zeros(nVectors,1);
for i = 1:nVectors
    for j = 1:numel(varargin)
        if ischar(varargin{j}) && strcmpi(varargin{j},vectorTitle{i})
            if order(i) > 0
                error('IDENTIFICATION:PLOT_VECTOR_COMPARISON:equalTitles','Two er more vectors have the same title.');
            end
            order(i) = j;
        end
    end
end
[~,ind] = sort(order,'ascend');
vectorTitle = flipud(vectorTitle(ind));
vector = cell(nVectors,1);
for i = 1:nVectors
    if ~isrealvector(p.Unmatched.(vectorTitle{i}))
        if ~isreal(p.Unmatched.(vectorTitle{i}))
            error('IDENTIFICATION:PLOT_VECTOR_COMPARISON:complexVector','The vector "%s" contains complex values.',vectorTitle{i});
        else
            error('IDENTIFICATION:PLOT_VECTOR_COMPARISON:invalidVector','The vector "%s" is not valid.',vectorTitle{i});
        end
    end
    vector{i} = p.Unmatched.(vectorTitle{i});
end

% validate vectors
if nVectors == 0
    error('IDENTIFICATION:PLOT_VECTOR_COMPARISON:missingVector','At least one vector has to be specified (in addition to the reference vector).');
end
if ~isvector(reference)
    error('IDENTIFICATION:PLOT_VECTOR_COMPARISON:notVector','The reference is not a vector.')
end
nElements = max(size(reference));
for i = 1:nVectors
    if ~isvector(vector{i})
        error('IDENTIFICATION:PLOT_VECTOR_COMPARISON:notVector','"%s" is not a vector.',vectorTitle{i});
    end
    if numel(vector{i}) ~= nElements
        error('IDENTIFICATION:PLOT_VECTOR_COMPARISON:elements','"%s" does not have the same number of elements as the reference vector.',vectorTitle{i});
    end
end

%% MAXIMUM AND MINIMUM VALUES
maxValue = max(reference);
minValue = min(reference);
for i = 1:nVectors
    maxValue = max(maxValue,max(vector{i}));
    minValue = min(minValue,min(vector{i}));
end
if ~isnan(minMaxValue) && maxValue < minMaxValue
    maxValue = minMaxValue;
end
if ~isnan(maxMinValue) && minValue > maxMinValue
    minValue = maxMinValue;
end
maxValue = maxValue*rangeScale;
minValue = minValue*rangeScale;

%% PLOT
units = 'centimeters';
fig = figure('name',figName,'units',units,'visible','off');
ax = axes('units',units);
hold on;
if includeTitle
    title(figName);
end

% x-axis (extend limits to next ticks)
set(ax,'xlim',[minValue maxValue]);
tmp = get(ax,'xtick');
dx = tmp(2)-tmp(1);
if maxValue > tmp(end)
    maxValue = tmp(end)+dx;
end
if minValue < tmp(1)
    minValue = tmp(1)-dx;
end
set(ax,'xlim',[minValue maxValue]);
xlabel(ax,valueLabel);

% y-axis
set(ax,'ylim',[0 nVectors]);
set(ax,'ytick',(1:nVectors)-0.5,'yticklabel',vectorTitle);

% grid
tmp = get(ax(1),'xtick');
plot([tmp(2:end)' tmp(2:end)'],[0 nVectors],'color',0.95*[1 1 1]);
for i = 1:nVectors
    plot([minValue maxValue],[i i],'color',0.6*[1 1 1]);
end

% plot
for i = 1:nVectors
    plot(reference,ones(nElements,1)*i-0.5,'ok','linewidth',1);
    plot(vector{i},ones(nElements,1)*i-0.5,'xr','linewidth',2);
end

% frame
plot([minValue minValue],[0 nVectors],'k',[minValue maxValue],[0 0],'k');
plot([maxValue maxValue],[0 nVectors],'k',[minValue maxValue],[0 0],'k');

% legend
if includeLegend
    axes('units',units,'visible','off');
    hold on;
    plot([0 0],[0 0],'ok','linewidth',1,'visible','off');
    plot([0 0],[0 0],'xr','linewidth',2,'visible','off');
    l = legend({legendReference,legendValue},'orientation','horizontal','units',units);
    lPos = get(l,'position');
else
    lPos = zeros(4,1);
end

%% RESIZE AND REPOSITION ELEMENTS
inset = get(ax,'tightinset');
axPos = [inset(1)+horizontalGap,inset(2)+lPos(4)+2*verticalGap,plotSize,nVectors*plotSize/15];
set(ax,'position',axPos);
if includeLegend
    set(l,'position',[axPos(1)+(axPos(3)-lPos(3))/2,verticalGap,lPos(3),lPos(4)]);
    set(l,'position',[axPos(1)+(axPos(3)-lPos(3))/2,verticalGap,lPos(3),lPos(4)]);
end
figW = plotSize+inset(1)+inset(3)+2*horizontalGap;
figH = nVectors*plotSize/15+inset(2)+inset(4)+lPos(4)+3*verticalGap;
set(fig,'position',[0,0,figW,figH]);

% show figure
movegui(fig,'center');
set(fig,'visible','on');