function printvectors(varargin)
%   PRINTVECTORS Prints one or more vectors.
%
%   REQUIRED INPUTS
%       'title',[vector]
%           One or more vectors to be compared to each other, given as
%           title/vector pairs.
%
%   OPTIONAL PARAMETERS
%       Key             | Value
%       -------------------------------------------------------------------
%       'header'        | Header printed before the vectors are printed.
%       'includeMean'   | Whether or not to include the mean value of each
%                       |     vector.
%       'includeAbsMean'| Whether or not to include the mean of the absolute
%                       |     values of each vector.
%
%   OUPUT
%       None.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% DEFAULT SETTINGS
header = '';
includeMean = false;
includeAbsMean = false;

%% PARSE AND VALIDATE INPUT
% validation functions
isbool = @(x) islogical(x) && isscalar(x);
isrealvector = @(x) isvector(x) && isnumeric(x) && isreal(x);

% build input scheme
p = inputParser;
p.FunctionName = 'printvectorcomparison';
p.CaseSensitive = false;
p.KeepUnmatched = true;
p.addParamValue('header',header,@ischar);
p.addParamValue('includeMean',includeMean,isbool);
p.addParamValue('includeAbsMean',includeAbsMean,isbool);
p.parse(varargin{:});

% get parameters
header = p.Results.header;
includeMean = p.Results.includeMean;
includeAbsMean = p.Results.includeAbsMean;

% get vectors (sorted by argument order)
vectorTitle = fieldnames(p.Unmatched);
nVectors = numel(vectorTitle);
order = zeros(nVectors,1);
for i = 1:nVectors
    for j = 1:numel(varargin)
        if ischar(varargin{j}) && strcmpi(varargin{j},vectorTitle{i})
            if order(i) > 0
                error('IDENTIFICATION:PRINT_VECTOR_COMPARISON:equalTitles','Two er more vectors have the same title.');
            end
            order(i) = j;
        end
    end
end
[~,ind] = sort(order,'ascend');
vectorTitle = vectorTitle(ind);
vector = cell(nVectors,1);
for i = 1:nVectors
    if ~isrealvector(p.Unmatched.(vectorTitle{i}))
        if ~isreal(p.Unmatched.(vectorTitle{i}))
            error('IDENTIFICATION:PRINT_VECTOR_COMPARISON:complexVector','The vector "%s" contains complex values.',vectorTitle{i});
        else
            error('IDENTIFICATION:PRINT_VECTOR_COMPARISON:invalidVector','The vector "%s" is not valid.',vectorTitle{i});
        end
    end
    vector{i} = p.Unmatched.(vectorTitle{i});
end

% validate vectors
if nVectors == 0
    error('IDENTIFICATION:PRINT_VECTOR_COMPARISON:missingVector','At least one vector has to be specified (in addition to the reference vector).');
end
nElements = max(size(vector{1}));
for i = 1:nVectors
    if ~isvector(vector{i})
        error('IDENTIFICATION:PRINT_VECTOR_COMPARISON:notVector','"%s" is not a vector.',vectorTitle{i});
    end
    if numel(vector{i}) ~= nElements
        error('IDENTIFICATION:PRINT_VECTOR_COMPARISON:elements','"%s" does not have the same number of elements as the first vector.',vectorTitle{i});
    end
end

%% PRINT
% ensure correct format
format = get(0,'format');
set(0,'format','short');
colWidth = 10;

% build full header
temp = zeros(nElements,nVectors);
str = blanks(nVectors*colWidth);
for i = 1:nVectors
    t = length(vectorTitle{i});
    if t > colWidth-1
        t = colWidth-1;
    end
    pos = (i-1)*colWidth+1;
    str(pos:pos+t-1) = vectorTitle{i}(1:t);
    temp(:,i) = vector{i};
end

% print all
if ~isempty(header)
    fprintf([header '\n\n\t' str '\n\n']);
end
disp(temp);
if includeMean && size(temp,1) > 1
    fprintf(['\t' repmat(['Mean' blanks(colWidth-5)],1,nVectors) '\n']);
    disp(mean(temp));
end
if includeAbsMean && size(temp,1) > 1
    fprintf(['\t' repmat(['Mean(abs)' blanks(colWidth-9)],1,nVectors) '\n']);
    disp(mean(abs(temp)));
end

% reset format
set(0,'format',format);