function displacements = simulatedisplacements(range,fsMeasurements,initialDisplacements,M,C,K,varargin)
%   SIMULATEDISPLACEMENTS Simulate displacements by the Newmark-Beta method.
%
%   REQUIRED INPUT
%       range
%           A vector containing the time interval of the portion of the
%           time series that will be returned.
%       fsMeasurements
%           Sampling rate of the simulated time series.
%       initialDisplacements
%           Initial displacements that are released when simulation begins.
%           If no load vector is applied, this results in free vibration
%           response.
%       M
%           Mass matrix of the simulated system.
%       C
%           Damping matrix of the simulated system.
%       K
%           Stiffness matrix of the simulated system.
%
%   OPTIONAL PARAMETERS
%       Key             | Value
%       -------------------------------------------------------------------
%       'fsSimulation'  | Sampling rate used during simulation. If
%                       |     necessary, time series are resampled to the 
%                       |     sampling rate given in fsMeasurements before
%                       |     they are returned.
%       'loadVector'    | Vector containing load amplitudes of each mode.
%                       |     The load itself will be generated randomly
%                       |     to obtain white noise load characteristics.
%       'gamma'         | Gamma value of the Newmark-Beta method.
%       'beta'          | Beta value of the Newmark-Beta method.
%
%   OUPUT
%       displacements
%           The simulated displacements. Each column constitutes a time
%           series.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% DEFAULT SETTINGS
fsSimulation = 1000;
loadVector = [0,0,0,0,0];

% linear acceleration method
gamma = 1/2;
beta = 1/6;

%% PARSE AND VALIDATE INPUT
% validation functions
isrealscalar = @(x) ~ischar(x) && isreal(x) && isscalar(x);
isnonnegativescalar0 = @(x) isrealscalar(x) && x >= 0;
isnonnegativescalar = @(x) isrealscalar(x) && x > 0;
isnonnegativevector2 = @(x) isvector(x) && all(arrayfun(isnonnegativescalar0,x)) && numel(x) == 2;
isnonnegativevector5 = @(x) isvector(x) && all(arrayfun(isnonnegativescalar0,x)) && numel(x) <= 5;
isrealmatrix = @(x) ismatrix(x) && isnumeric(x) && isreal(x);
isrealmatrix5 = @(x) isrealmatrix(x) && size(x,1) == 5 && size(x,2) == 5;

% build input scheme
p = inputParser;
p.FunctionName = 'simulatedisplacements';
p.CaseSensitive = false;
p.KeepUnmatched = true;
p.addRequired('range',isnonnegativevector2);
p.addRequired('fsMeasurements',isnonnegativescalar);
p.addRequired('initialDisplacements',isnonnegativevector5);
p.addRequired('M',isrealmatrix5);
p.addRequired('C',isrealmatrix5);
p.addRequired('K',isrealmatrix5);
p.addParamValue('fsSimulation',fsSimulation,isnonnegativescalar);
p.addParamValue('loadVector',loadVector,isnonnegativevector5);
p.addParamValue('gamma',gamma,isnonnegativescalar);
p.addParamValue('beta',beta,isnonnegativescalar);
p.parse(range,fsMeasurements,initialDisplacements,M,C,K,varargin{:});

% get parameters
fsSimulation = p.Results.fsSimulation;
loadVector = p.Results.loadVector;
gamma = p.Results.gamma;
beta = p.Results.beta;

% validate parameters and set values based on given arguments
nSamples = range(2)*fsSimulation+1;
dt = 1/fsSimulation;
nDofs = size(M,1);
if isrow(initialDisplacements)
    initialDisplacements = initialDisplacements';
end

%% LOAD (WHITE NOISE)
R = diag(loadVector)*rand(nDofs,nSamples);

%% CALCULATION OF RESPONSE (NEWMARK-BETA METHOD)
% setup
u = zeros(nDofs,nSamples);
v = zeros(nDofs,nSamples);
a = zeros(nDofs,nSamples);

% initial conditions
u(:,1) = initialDisplacements;
v(:,1) = zeros(nDofs,1);
a(:,1) = M\(R(:,1)-C*v(:,1)-K*u(:,1));

% effective stiffness matrix
K_eff = M/(beta*dt^2)+C*gamma/(beta*dt)+K;

% calculations for each time step
tmp1 = M/(beta*dt)+C*gamma/beta;
tmp2 = M/(2*beta)+C*dt*(gamma/(2*beta)-1);
for i = 1:nSamples-1
    % effective resiual
    dR_eff = R(:,i+1)-R(:,i)+tmp1*v(:,i)+tmp2*a(:,i);
    
    % update
    du = K_eff\dR_eff;
    u(:,i+1) = du+u(:,i);
    v(:,i+1) = gamma/(beta*dt)*du-(gamma/beta-1)*v(:,i)-dt*(gamma/(2*beta)-1)*a(:,i);
    a(:,i+1) = 1/(beta*dt^2)*(du-dt*v(:,i))-(1/(2*beta)-1)*a(:,i);
end

% select range only
u = u(:,range(1)*fsSimulation+1:end);

% resample to desired sampling frequency
displacements = resample(u',fsMeasurements,fsSimulation);