function K = stiffnessmatrix(r,Props,modal)
%   STIFFNESSMATRIX Calculates the stiffness matrix for a given
%   configuration of the global degrees of freedom.
%
%   For more information, see "Appendix A - Stiffness matrix".
%
%   INPUT
%       r
%           Vector that contains the static displacements of the five
%           degrees of freedom:
%           r(1): Horizontal displacement (translation), given in meters.
%           r(2): Rotation in the horizontal plane, given in radians.
%           r(3): Vertical displacement (translation), given in meters.
%           r(4): Rotation in the vertical plane, given in radians.
%           r(5): Rotation about the longitudinal axis of the section model, given in radians.
%       Props
%           Structure containing all properties of the rig.
%           See also GETRIGPROPERTIES, GETABAQUSPROPERTIES.
%       modal
%           Boolean determining whether or not the returned matrix should
%           be modal.
%
%   OUTPUT
%       The stiffness matrix of the given configuration.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Author: Lars Halvor Kaasa
%   Copyright: Norwegian University of Science and Technology
%   Date: June 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% STIFFNESS MATRIX OF FULL SYSTEM
hStiffnessRatio = 0.5;                                      % approximate ratio between stiffness from horizontal springs and total stiffness
bv = Props.distanceCentreOfMassToVSprings;
bh = bv+hStiffnessRatio*(Props.distanceCentreOfMassToHSprings-bv);

% stiffness matrices of the basic suspension (sub)systems
K1 = subK(1,r(1)+r(2)*bh,r(3)+r(4)*bv,r(5),Props);
K2 = subK(2,r(1)-r(2)*bh,r(3)-r(4)*bv,r(5),Props);

% transformation from subsystems to full system
T = [
    1   0   0   1   0   0
    bh  0   0   -bh 0   0
    0   1   0   0   1   0
    0   bv  0   0   -bv 0
    0   0   1   0   0   1
    ];
K = T*[K1,zeros(3);zeros(3),K2]*T';

% if required, transform to modal matrix
if modal
    [vectors,values] = eig(K,Props.M);
    values = diag(values);
    [~,sortedPos] = sort(values,'ascend');
    modeShapes = vectors(:,sortedPos);
    K = modeShapes'*K*modeShapes;
end

%% STIFFNESS MATRIX OF A BASIC SUSPENSION SYSTEM
    function subK = subK(side,rx,rz,rt,Props)
        a = Props.distanceLongitudinalAxisToVSprings;
        e = Props.wireBracketDiameter/2;
        h = Props.rigHeight/2-e;
        w = Props.rigWidth/2-e;
        mg = Props.suspendedMass/2;
        z = Props.cgDeviationZ;
        
        % suspension lengths
        l0 = [w,w,h,h,h,h]';
        lx = zeros(6,1);                                    % x-components
        lx(1) = w+rx;
        lx(2) = w-rx;
        lx(3:4) = rx-rt*e;
        lx(5:6) = rx+rt*e;
        lx = abs(lx);
        lz = zeros(6,1);                                    % z-components
        lz(1) = rz+rt*e;
        lz(2) = rz-rt*e;
        lz(3) = h+rz+rt*a;
        lz(4) = h+rz-rt*a;
        lz(5) = h-rz-rt*a;
        lz(6) = h-rz+rt*a;
        lz = abs(lz);
        l = sqrt(lx.^2+lz.^2);                              % full lengths
        
        % spring stiffness
        k = zeros(6,1);
        if side == 1
            indexes = 1:2;
        elseif side == 2
            indexes = 3:4;
        end
        k(1:2) = Props.hEffectiveStiffness(indexes);
        k(3:4) = Props.uvEffectiveStiffness(indexes);
        k(5:6) = Props.lvEffectiveStiffness(indexes);
        
        % static forces
        f0 = zeros(6,1);
        if side == 1
            indexes = 3:4;
        elseif side == 2
            indexes = 5:6;
        end
        f0(1:2) = Props.initialForce(side);
        f0(3:4) = Props.initialForce(indexes)+Props.suspendedMass*9.81/4;
        f0(5:6) = Props.initialForce(indexes);
        f0 = f0+k.*(l-l0);
        f0x = f0.*lx./l;
        f0z = f0.*lz./l;
        
        % participation factors
        pfx = [1,-1,1,1,1,1]';
        pfz = [1,1,1,1,-1,-1]';
        pfg = [-1,1,-1,-1,1,1]';
        pftx = zeros(6,1);
        pftx([1,2]) = [1,1]*rt*e;
        pftx([4,5]) = [-1,1]*(rt*a+e);
        pftx([3,6]) = [1,-1]*(rt*a-e);
        pftz = zeros(6,1);
        pftz([1,2]) = [1,-1]*e;
        pftz([4,5]) = [-1,-1]*(a-rt*e);
        pftz([3,6]) = [1,1]*(a+rt*e);

        % stiffness due to horizontal displacement
        kx = pfx.*k.*lx./l;                                 % effective stiffness in the wire direction
        kxx = kx.*lx./l;                                    % x-components
        kxz = kx.*lz./l;                                    % z-components
        gx = f0.*lz./(l.^2);                                % geometric stiffness perpendicular to the wire direction
        gxx = gx.*lz./l;                                    % x-components
        gxz = gx.*lx./l;                                    % z-components
        
        % stiffness due to vertical displacement
        kz = pfz.*k.*lz./l;                                 % effective stiffness in the wire direction
        kzx = kz.*lx./l;                                    % x-components
        kzz = kz.*lz./l;                                    % z-components
        gz = f0.*lx./(l.^2);                                % geometric stiffness perpendicular to the wire direction
        gzx = gz.*lz./l;                                    % x-components
        gzz = gz.*lx./l;                                    % z-components
        
        % stiffness due to rotation
        kt = (pftz.*lz+pftx.*lx).*k./l;
        ktx = kt.*lx./l;                                    % x-components of effective stiffness
        ktz = kt.*lz./l;                                    % z-components of effective stiffness
        gt = gx.*pftx-gz.*pftz;
        gtx = gt.*lz./l;
        gtz = gt.*lx./l;
        gfstatic = sum(f0x.*[e,e,a,-a,a,-a]'+f0z.*[0,0,e,e,e,e]');
        gmass = mg*z;
        
        % matrix assembly
        subK = zeros(3);
        subK(1,1) = sum(pfx.*kxx)+sum(gxx);                 % stiffness due to horizontal displacement
        subK(2,1) = sum(pfz.*kxz)+sum(pfg.*gxz);
        subK(3,1) = sum(pftx.*kxx+pftz.*kxz)+sum((gxx.*pftx-gxz.*pftz).*pfx);
        subK(1,2) = sum(pfx.*kzx)+sum(pfg.*gzx);            % stiffness due to vertical displacement
        subK(2,2) = sum(pfz.*kzz)+sum(gzz);
        subK(3,2) = sum(pftx.*kzx+pftz.*kzz)+sum((gzz.*pftz-gzx.*pftx).*pfz);
        subK(1,3) = sum(pfx.*ktx)+sum(pfx.*gtx);            % stiffness due to rotation
        subK(2,3) = sum(pfz.*ktz)+sum(-pfz.*gtz);
        subK(3,3) = sum(pftx.*ktx+pftz.*ktz)+sum(gtx.*pftx-gtz.*pftz)+gfstatic+gmass;
    end
end